--- avail.c-orig	2007-01-02 19:50:42.000000000 -0700
+++ avail.c	2007-01-30 13:53:34.000000000 -0700
@@ -58,7 +58,7 @@
 #define CSV_OUTPUT              1
 
 
-/* archived state types */
+/* archived-state entry types */
 #define AS_CURRENT_STATE        -1   /* special case for initial assumed state */
 #define AS_NO_DATA		0
 #define AS_PROGRAM_END		1
@@ -76,6 +76,7 @@
 #define AS_HOST_DOWNTIME_START  12
 #define AS_HOST_DOWNTIME_END    13
 
+/* archived-state state types */
 #define AS_SOFT_STATE           1
 #define AS_HARD_STATE           2
 
@@ -108,6 +109,23 @@
 #define TIMEPERIOD_LAST7DAYS	12
 #define TIMEPERIOD_LAST31DAYS	13
 
+static char *TimePeriods[] = {
+	"custom",
+	"today",
+	"yesterday",
+	"thisweek",
+	"lastweek",
+	"thismonth",
+	"lastmonth",
+	"thisquarter",
+	"lastquarter",
+	"thisyear",
+	"lastyear",
+	"last24hours",
+	"last7days",
+	"last31days"
+};
+
 #define MIN_TIMESTAMP_SPACING	10
 
 #define MAX_ARCHIVE_SPREAD	65
@@ -126,6 +144,63 @@
 	struct archived_state_struct *next;
         }archived_state;
 
+typedef struct _sg_summary_total_ {
+	double average_percent_time_up;
+	double average_percent_time_up_known;
+	double average_percent_time_down;
+	double average_percent_time_down_known;
+	double average_percent_time_unreachable;
+	double average_percent_time_unreachable_known;
+	double average_percent_time_ok;
+	double average_percent_time_ok_known;
+	double average_percent_time_unknown;
+	double average_percent_time_unknown_known;
+	double average_percent_time_warning;
+	double average_percent_time_warning_known;
+	double average_percent_time_critical;
+	double average_percent_time_critical_known;
+	double average_percent_time_indeterminate;
+	double sla_actual;
+	double sla_target;
+} GROUP_SUMMARY_TOTALS;
+
+typedef struct _sg_stats_ {
+	servicegroup *sg;
+	double average_percent_time_up;
+	double average_percent_time_up_known;
+	double average_percent_time_down;
+	double average_percent_time_down_known;
+	double average_percent_time_unreachable;
+	double average_percent_time_unreachable_known;
+	double average_percent_time_ok;
+	double average_percent_time_ok_known;
+	double average_percent_time_unknown;
+	double average_percent_time_unknown_known;
+	double average_percent_time_warning;
+	double average_percent_time_warning_known;
+	double average_percent_time_critical;
+	double average_percent_time_critical_known;
+	double average_percent_time_indeterminate;
+	double sla_actual;
+	double sla_target;
+} servicegroup_stats;
+
+#define SG_SORT_NAME	0
+#define SG_SORT_OK	1
+#define SG_SORT_WARN	2
+#define SG_SORT_UNK	3
+#define SG_SORT_CRIT	4
+#define SG_SORT_UND	5
+#define SG_SORT_SLAA	6
+#define SG_SORT_SLAT	7
+
+#define SORT_DIR_ASC	0
+#define SORT_DIR_DESC	1
+
+#define MAX_SG_LIMIT	4096
+
+static servicegroup_stats sg_stat_array[MAX_SG_LIMIT];
+
 typedef struct avail_subject_struct{
 	int type;
 	char *host_name;
@@ -200,17 +275,24 @@
 int show_all_servicegroups=FALSE;
 int show_all_services=FALSE;
 
+int show_hostgroups_summary=FALSE;
+int show_servicegroups_summary=FALSE;
+
 int assume_initial_states=TRUE;
 int assume_state_retention=TRUE;
 int assume_states_during_notrunning=TRUE;
-int initial_assumed_host_state=AS_NO_DATA;
-int initial_assumed_service_state=AS_NO_DATA;
+int initial_assumed_host_state=AS_HOST_UP;
+int initial_assumed_service_state=AS_SVC_OK;
 int include_soft_states=FALSE;
+int sort_column=0;
+int sort_direction=0;
 
 char *hostgroup_name="";
 char *host_name="";
 char *servicegroup_name="";
 char *svc_description="";
+static char *remote_user="";
+static userEntry *userRoot=NULL;
 
 void create_subject_list(void);
 void add_subject(int,char *,char *);
@@ -223,8 +305,12 @@
 void compute_subject_downtime_part_times(time_t,time_t,int,avail_subject *);
 void display_hostgroup_availability(void);
 void display_specific_hostgroup_availability(hostgroup *);
+void display_hostgroup_availability_summary(void);
+int display_specific_hostgroup_availability_summary(hostgroup *, char *, GROUP_SUMMARY_TOTALS *);
 void display_servicegroup_availability(void);
 void display_specific_servicegroup_availability(servicegroup *);
+void display_servicegroup_availability_summary(void);
+int display_specific_servicegroup_availability_summary(servicegroup *, servicegroup_stats *);
 void display_host_availability(void);
 void display_service_availability(void);
 void write_log_entries(avail_subject *);
@@ -233,6 +319,8 @@
 
 void host_report_url(char *,char *);
 void service_report_url(char *,char *,char *);
+void service_detail_url(char *, char *);
+void avail_sort_url(char *, char *, int);
 void compute_report_times(void);
 
 int convert_host_state_to_archived_state(int);
@@ -243,7 +331,7 @@
 void free_availability_data(void);
 void free_archived_state_list(archived_state *);
 void read_archived_state_data(void);
-void scan_log_file_for_archived_state_data(char *);
+void scan_log_file_for_archived_state_data(char *,int);
 void convert_timeperiod_to_times(int);
 unsigned long calculate_total_time(time_t,time_t);
 
@@ -256,6 +344,7 @@
 int backtrack_archives=2;
 int earliest_archive=0;
 
+int debugging=FALSE;
 int embedded=FALSE;
 int display_header=TRUE;
 
@@ -263,6 +352,8 @@
 
 int output_format=HTML_OUTPUT;
 
+static void show_subj_time (unsigned long t);
+static void dump_subjects (void);
 
 
 int main(int argc, char **argv){
@@ -314,6 +405,16 @@
 		return ERROR;
                 }
 
+#ifdef DEBUG_BOB
+	printf("Service Group         Alias                  SLA%%\n");
+	printf("--------------------  --------------------  ------\n");
+	servicegroup *ts;
+	for (ts=servicegroup_list; ts != NULL; ts=ts->next)
+	{
+		printf("%-20.20s  %-20.20s  %6.3f\n", ts->group_name, ts->alias, ts->sla_target);
+	}
+#endif
+
 	/* read all status data */
 	result=read_all_status_data(get_cgi_config_location(),READ_ALL_STATUS_DATA);
 	if(result==ERROR){
@@ -353,8 +454,13 @@
 	document_header(TRUE);
 
 	/* get authentication information */
+	memset(&current_authdata, 0, sizeof(current_authdata));
+	current_authdata.username = remote_user;	/* remote_user is set by process_cgivars() */
 	get_authentication_information(&current_authdata);
 
+	/* create a list of valid remote users */
+	populateUserList(&userRoot, 1);
+
 
 	if(compute_time_from_parts==TRUE)
 		compute_report_times();
@@ -502,6 +608,7 @@
 
 			printf("<form method=\"GET\" action=\"%s\">\n",AVAIL_CGI);
 
+			printf("<input type='hidden' name='username' value='%s'>\n",remote_user);
 			printf("<input type='hidden' name='t1' value='%lu'>\n",(unsigned long)t1);
 			printf("<input type='hidden' name='t2' value='%lu'>\n",(unsigned long)t2);
 			if(show_log_entries==TRUE)
@@ -521,6 +628,8 @@
 			printf("<input type='hidden' name='assumestateretention' value='%s'>\n",(assume_state_retention==TRUE)?"yes":"no");
 			printf("<input type='hidden' name='assumestatesduringnotrunning' value='%s'>\n",(assume_states_during_notrunning==TRUE)?"yes":"no");
 			printf("<input type='hidden' name='includesoftstates' value='%s'>\n",(include_soft_states==TRUE)?"yes":"no");
+			printf("<input type='hidden' name='sortcol' value='%d'>\n",sort_column);
+			printf("<input type='hidden' name='sortdir' value='%d'>\n",sort_direction);
 
 			printf("<tr><td valign=top align=left class='optBoxItem'>First assumed %s state:</td><td valign=top align=left class='optBoxItem'>%s</td></tr>\n",(display_type==DISPLAY_SERVICE_AVAIL)?"service":"host",(display_type==DISPLAY_HOST_AVAIL || display_type==DISPLAY_HOSTGROUP_AVAIL || display_type==DISPLAY_SERVICEGROUP_AVAIL)?"First assumed service state":"");
 			printf("<tr>\n");
@@ -583,7 +692,13 @@
 			printf("</td>\n");
 			printf("</tr>\n");
 
-			printf("<tr><td valign=top align=left></td>\n");
+			printf("<tr><td valign=top align=left class='optBoxItem'>User View:</td><td valign=top align=left class='optBoxItem'></td></tr>\n");
+			printf("<tr>\n");
+			printf("<td valign=top align=left class='optBoxItem'>\n");
+			printf("<select name='username'>\n");
+			userListDropDown(userRoot, remote_user);
+			printf("</select>\n");
+			printf("</td>\n");
 			printf("<td valign=top align=left CLASS='optBoxItem'>\n");
 			printf("<input type='submit' value='Update'>\n");
 			printf("</td>\n");
@@ -655,6 +770,15 @@
 		printf("<table border=0 cellpadding=5>\n");
 
 		printf("<tr>");
+		printf("<td valign=top class='reportSelectSubTitle'>User View:</td>\n");
+		printf("<td valign=top align=left class='optBoxItem'>\n");
+		printf("<select name='username'>\n");
+		userListDropDown(userRoot, remote_user);
+		printf("</select>\n");
+		printf("</td>\n");
+		printf("</tr>\n");
+
+		printf("<tr>");
 		printf("<td valign=top class='reportSelectSubTitle'>Report Period:</td>\n");
 		printf("<td valign=top align=left class='optBoxItem'>\n");
 		printf("<select name='timeperiod'>\n");
@@ -779,7 +903,7 @@
 			printf("<select name='initialassumedhoststate'>\n");
 			printf("<option value=%d>Unspecified\n",AS_NO_DATA);
 			printf("<option value=%d>Current State\n",AS_CURRENT_STATE);
-			printf("<option value=%d>Host Up\n",AS_HOST_UP);
+			printf("<option value=%d SELECTED>Host Up\n",AS_HOST_UP);
 			printf("<option value=%d>Host Down\n",AS_HOST_DOWN);
 			printf("<option value=%d>Host Unreachable\n",AS_HOST_UNREACHABLE);
 			printf("</select>\n");
@@ -791,7 +915,7 @@
 		printf("<select name='initialassumedservicestate'>\n");
 		printf("<option value=%d>Unspecified\n",AS_NO_DATA);
 		printf("<option value=%d>Current State\n",AS_CURRENT_STATE);
-		printf("<option value=%d>Service Ok\n",AS_SVC_OK);
+		printf("<option value=%d SELECTED>Service Ok\n",AS_SVC_OK);
 		printf("<option value=%d>Service Warning\n",AS_SVC_WARNING);
 		printf("<option value=%d>Service Unknown\n",AS_SVC_UNKNOWN);
 		printf("<option value=%d>Service Critical\n",AS_SVC_CRITICAL);
@@ -803,6 +927,52 @@
 		printf("<input type='text' name='backtrack' size='2' maxlength='2' value='%d'>\n",backtrack_archives);
 		printf("</td></tr>\n");
 
+		printf("<tr><td class='reportSelectSubTitle' align=right>Sort by Column:</td>\n");
+		printf("<td class='reportSelectItem'>\n");
+		printf("<select name='sortcol'>\n");
+                if (display_type==DISPLAY_HOSTGROUP_AVAIL) {
+			printf("<option value=0 SELECTED>Host Group\n");
+			printf("<option value=1>Time Up\n");
+			printf("<option value=2>Time Down\n");
+			printf("<option value=3>Time Unreachable\n");
+			printf("<option value=4>Time Undetermined\n");
+		}
+                if (display_type==DISPLAY_HOST_AVAIL) {
+			printf("<option value=0 SELECTED>Host\n");
+			printf("<option value=1>Time Up\n");
+			printf("<option value=2>Time Down\n");
+			printf("<option value=3>Time Unreachable\n");
+			printf("<option value=4>Time Undetermined\n");
+		}
+                if (display_type==DISPLAY_SERVICE_AVAIL) {
+			printf("<option value=0 SELECTED>Service\n");
+			printf("<option value=1>Time Ok\n");
+			printf("<option value=2>Time Warning\n");
+			printf("<option value=3>Time Unknown\n");
+			printf("<option value=4>Time Critical\n");
+			printf("<option value=5>Time Undetermined\n");
+		}
+                if (display_type==DISPLAY_SERVICEGROUP_AVAIL) {
+			printf("<option value=0>Service Group\n");
+			printf("<option value=1>Time Ok\n");
+			printf("<option value=2>Time Warning\n");
+			printf("<option value=3>Time Unknown\n");
+			printf("<option value=4>Time Critical\n");
+			printf("<option value=5>Time Undetermined\n");
+			printf("<option value=6 SELECTED>Actual SLA\n");
+			printf("<option value=7>Target SLA\n");
+		}
+		printf("</select>\n");
+		printf("</td></tr>\n");
+
+		printf("<tr><td class='reportSelectSubTitle' align=right>Sort Direction:</td>\n");
+		printf("<td class='reportSelectItem'>\n");
+		printf("<select name='sortdir'>\n");
+		printf("<option value=0>Ascending\n");
+		printf("<option value=1 SELECTED>Descending\n");
+		printf("</select>\n");
+		printf("</td></tr>\n");
+
 		if((display_type==DISPLAY_HOST_AVAIL && show_all_hosts==TRUE) || (display_type==DISPLAY_SERVICE_AVAIL && show_all_services==TRUE)){
 			printf("<tr>");
 			printf("<td valign=top class='reportSelectSubTitle'>Output in CSV Format:</td>\n");
@@ -828,12 +998,14 @@
 		printf("<p><div align=center>\n");
 
 	        printf("<form method=\"get\" action=\"%s\">\n",AVAIL_CGI);
+		printf("<input type='hidden' name='username' value='%s'>\n",remote_user);
 		printf("<input type='hidden' name='get_date_parts'>\n");
 
 		printf("<table border=0 cellpadding=5>\n");
 
 		printf("<tr><td class='reportSelectSubTitle' valign=center>Hostgroup(s):</td><td align=left valign=center class='reportSelectItem'>\n");
 		printf("<select name='hostgroup'>\n");
+		printf("<option value='summary'>** HOSTGROUP SUMMARY **\n");
 		printf("<option value='all'>** ALL HOSTGROUPS **\n");
 		for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next){
 			if(is_authorized_for_hostgroup(temp_hostgroup,&current_authdata)==TRUE)
@@ -858,6 +1030,7 @@
 		printf("<p><div align=center>\n");
 
 	        printf("<form method=\"get\" action=\"%s\">\n",AVAIL_CGI);
+		printf("<input type='hidden' name='username' value='%s'>\n",remote_user);
 		printf("<input type='hidden' name='get_date_parts'>\n");
 
 		printf("<table border=0 cellpadding=5>\n");
@@ -890,12 +1063,14 @@
 		printf("<p><div align=center>\n");
 
 	        printf("<form method=\"get\" action=\"%s\">\n",AVAIL_CGI);
+		printf("<input type='hidden' name='username' value='%s'>\n",remote_user);
 		printf("<input type='hidden' name='get_date_parts'>\n");
 
 		printf("<table border=0 cellpadding=5>\n");
 
 		printf("<tr><td class='reportSelectSubTitle' valign=center>Servicegroup(s):</td><td align=left valign=center class='reportSelectItem'>\n");
 		printf("<select name='servicegroup'>\n");
+		printf("<option value='summary'>** SERVICEGROUP SUMMARY **\n");
 		printf("<option value='all'>** ALL SERVICEGROUPS **\n");
 		for(temp_servicegroup=servicegroup_list;temp_servicegroup!=NULL;temp_servicegroup=temp_servicegroup->next){
 			if(is_authorized_for_servicegroup(temp_servicegroup,&current_authdata)==TRUE)
@@ -939,6 +1114,7 @@
 		printf("<p><div align=center>\n");
 
 	        printf("<form method=\"get\" action=\"%s\" name='serviceform'>\n",AVAIL_CGI);
+		printf("<input type='hidden' name='username' value='%s'>\n",remote_user);
 		printf("<input type='hidden' name='get_date_parts'>\n");
 		printf("<input type='hidden' name='host' value='%s'>\n",(firsthostpointer==NULL)?"unknown":firsthostpointer);
 
@@ -1001,9 +1177,11 @@
 			if(output_format==HTML_OUTPUT){
 				get_time_breakdown((time_t)(report_end_time-report_start_time),&days,&hours,&minutes,&seconds);
 				printf("<div align=center class='reportTime'>[ Availability report completed in %d min %d sec ]</div>\n",minutes,seconds);
-				printf("<BR><BR>\n");
+				printf("<BR>\n");
 			        }
 
+			if (debugging) dump_subjects();
+
 			/* display availability data */
 			if(display_type==DISPLAY_HOST_AVAIL)
 				display_host_availability();
@@ -1028,15 +1206,16 @@
 		printf("<p><div align=center>\n");
 
 	        printf("<form method=\"get\" action=\"%s\">\n",AVAIL_CGI);
+		printf("<input type='hidden' name='username' value='%s'>\n",remote_user);
 
 		printf("<table border=0 cellpadding=5>\n");
 
 		printf("<tr><td class='reportSelectSubTitle' align=right>Type:</td>\n");
 		printf("<td class='reportSelectItem'>\n");
 		printf("<select name='report_type'>\n");
+		printf("<option value=servicegroups>Servicegroup(s)\n");
 		printf("<option value=hostgroups>Hostgroup(s)\n");
 		printf("<option value=hosts>Host(s)\n");
-		printf("<option value=servicegroups>Servicegroup(s)\n");
 		printf("<option value=services>Service(s)\n");
 		printf("</select>\n");
 		printf("</td></tr>\n");
@@ -1159,6 +1338,10 @@
 				strcpy(hostgroup_name,variables[x]);
 			display_type=DISPLAY_HOSTGROUP_AVAIL;
 			show_all_hostgroups=(strcmp(hostgroup_name,"all"))?FALSE:TRUE;
+			if (!strcmp(hostgroup_name,"summary")) {
+				show_all_hostgroups=TRUE;
+				show_hostgroups_summary=TRUE;
+				}
 		        }
 
 		/* we found the servicegroup argument */
@@ -1176,6 +1359,10 @@
 				strcpy(servicegroup_name,variables[x]);
 			display_type=DISPLAY_SERVICEGROUP_AVAIL;
 			show_all_servicegroups=(strcmp(servicegroup_name,"all"))?FALSE:TRUE;
+			if (!strcmp(servicegroup_name,"summary")) {
+				show_all_servicegroups=TRUE;
+				show_servicegroups_summary=TRUE;
+				}
 		        }
 
 		/* we found the host argument */
@@ -1378,6 +1565,10 @@
 			compute_time_from_parts=FALSE;
 		        }
 
+		/* we found the debug option  */
+		else if(!strcmp(variables[x],"debug"))
+			debugging=TRUE;
+
 		/* we found the embed option */
 		else if(!strcmp(variables[x],"embedded"))
 			embedded=TRUE;
@@ -1645,8 +1836,49 @@
 			        }
 		        }
 
+		/* we found the column sort-order option */
+		else if(!strcmp(variables[x],"sortcol")){
+			x++;
+			if(variables[x]==NULL){
+				error=TRUE;
+				break;
+			        }
+
+			sort_column=atoi(variables[x]);
+		        }
+
+		/* we found the column sort-direction option */
+		else if(!strcmp(variables[x],"sortdir")){
+			x++;
+			if(variables[x]==NULL){
+				error=TRUE;
+				break;
+			        }
+
+			sort_direction=atoi(variables[x]);
+		        }
+
+		/* we found the service description argument */
+		else if(!strcmp(variables[x],"username")){
+			x++;
+			if(variables[x]==NULL){
+				error=TRUE;
+				break;
+			        }
+
+			remote_user=(char *)malloc(strlen(variables[x])+1);
+			if(remote_user==NULL)
+				remote_user="";
+			else
+				strcpy(remote_user,variables[x]);
+		        }
+
 	        }
 
+	/* If username was not specified, default to REMOTE_USER environment value */
+	if ((!remote_user || !remote_user[0]) && getenv("REMOTE_USER") != NULL)
+		remote_user = getenv("REMOTE_USER");
+		
 	/* free memory allocated to the CGI variables */
 	free_cgivars(variables);
 
@@ -2875,7 +3107,7 @@
 #endif
 
 		/* scan the log file for archived state data */
-		scan_log_file_for_archived_state_data(filename);
+		scan_log_file_for_archived_state_data(filename, (current_archive == oldest_archive));
 	        }
 
 	return;
@@ -2884,14 +3116,14 @@
 
 
 /* grabs archives state data from a log file */
-void scan_log_file_for_archived_state_data(char *filename){
+void scan_log_file_for_archived_state_data(char *filename, int isLastLog){
 	char *input=NULL;
 	char *input2=NULL;
 	char entry_host_name[MAX_INPUT_BUFFER];
 	char entry_svc_description[MAX_INPUT_BUFFER];
 	char *plugin_output;
 	char *temp_buffer;
-	time_t time_stamp;
+	time_t time_stamp, first_stamp;
 	mmapfile *thefile;
 	avail_subject *temp_subject;
 	int state_type;
@@ -2899,13 +3131,19 @@
 	if((thefile=mmap_fopen(filename))==NULL)
 		return;
 
+	first_stamp = 0;
+
 	while(1){
 
 		/* free memory */
-		free(input);
-		free(input2);
-		input=NULL;
-		input2=NULL;
+		if (input) {
+			free(input);
+			input=NULL;
+			}
+		if (input2) {
+			free(input2);
+			input2=NULL;
+			}
 
 		/* read the next line */
 		if((input=mmap_fgets(thefile))==NULL)
@@ -2919,6 +3157,8 @@
 		temp_buffer=my_strtok(input2,"]");
 		time_stamp=(temp_buffer==NULL)?(time_t)0:(time_t)strtoul(temp_buffer+1,NULL,10);
 
+		if (!first_stamp) first_stamp = time_stamp;
+
 		/* program starts/restarts */
 		if(strstr(input," starting..."))
 			add_global_archived_state(AS_PROGRAM_START,AS_NO_DATA,time_stamp,"Program start");
@@ -2949,9 +3189,12 @@
 
 				/* state types */
 				if(strstr(input,";SOFT;")){
-					if(include_soft_states==FALSE)
+					if(strstr(input,";RECOVERY") || strstr(input,";UP;"))
+						state_type=AS_HARD_STATE;
+					else if(include_soft_states==TRUE)
+						state_type=AS_SOFT_STATE;
+					else
 						continue;
-					state_type=AS_SOFT_STATE;
 				        }
 				if(strstr(input,";HARD;"))
 					state_type=AS_HARD_STATE;
@@ -2993,7 +3236,30 @@
 					add_scheduled_downtime(AS_HOST_DOWNTIME_START,time_stamp,temp_subject);
 				else
 					add_scheduled_downtime(AS_HOST_DOWNTIME_END,time_stamp,temp_subject);
+			        }
+
+			/* reminder downtime notices */
+			else if(strstr(input,"HOST DOWNTIME STATE:")){
+
+				/* get host name */
+				temp_buffer=my_strtok(NULL,":");
+				temp_buffer=my_strtok(NULL,";");
+				strncpy(entry_host_name,(temp_buffer==NULL)?"":temp_buffer+1,sizeof(entry_host_name));
+				entry_host_name[sizeof(entry_host_name)-1]='\x0';
+
+				/* see if there is a corresponding subject for this host */
+				temp_subject=find_subject(HOST_SUBJECT,entry_host_name,NULL);
+				if(temp_subject==NULL)
+					continue;
 
+				if(show_scheduled_downtime==FALSE)
+					continue;
+			
+				if(strstr(input,";STARTED;")) {
+					if (isLastLog && time_stamp == first_stamp) {
+						add_scheduled_downtime(AS_HOST_DOWNTIME_START,time_stamp,temp_subject);
+						}
+					}
 			        }
 		        }
 
@@ -3020,9 +3286,12 @@
 
 				/* state types */
 				if(strstr(input,";SOFT;")){
-					if(include_soft_states==FALSE)
+					if(strstr(input,";RECOVERY;") || strstr(input,";OK;"))
+						state_type=AS_HARD_STATE;
+					else if(include_soft_states==TRUE)
+						state_type=AS_SOFT_STATE;
+					else
 						continue;
-					state_type=AS_SOFT_STATE;
 				        }
 				if(strstr(input,";HARD;"))
 					state_type=AS_HARD_STATE;
@@ -3074,6 +3343,35 @@
 					add_scheduled_downtime(AS_SVC_DOWNTIME_END,time_stamp,temp_subject);
 		                }
 
+			/* reminder service downtime notices */
+			else if(strstr(input,"SERVICE DOWNTIME STATE:")){
+
+				/* get host name */
+				temp_buffer=my_strtok(NULL,":");
+				temp_buffer=my_strtok(NULL,";");
+				strncpy(entry_host_name,(temp_buffer==NULL)?"":temp_buffer+1,sizeof(entry_host_name));
+				entry_host_name[sizeof(entry_host_name)-1]='\x0';
+
+				/* get service description */
+				temp_buffer=my_strtok(NULL,";");
+				strncpy(entry_svc_description,(temp_buffer==NULL)?"":temp_buffer,sizeof(entry_svc_description));
+				entry_svc_description[sizeof(entry_svc_description)-1]='\x0';
+
+				/* see if there is a corresponding subject for this service */
+				temp_subject=find_subject(SERVICE_SUBJECT,entry_host_name,entry_svc_description);
+				if(temp_subject==NULL)
+					continue;
+
+				if(show_scheduled_downtime==FALSE)
+					continue;
+			
+				if(strstr(input,";STARTED;")) {
+					if (isLastLog && time_stamp == first_stamp) {
+						add_scheduled_downtime(AS_SVC_DOWNTIME_START,time_stamp,temp_subject);
+						}
+					}
+		                }
+
 			/* scheduled host downtime notices */
 			else if(strstr(input,"HOST DOWNTIME ALERT:")){
 
@@ -3101,13 +3399,42 @@
 						add_scheduled_downtime(AS_HOST_DOWNTIME_END,time_stamp,temp_subject);
 				        }
 			        }
+
+			/* scheduled host downtime notices */
+			else if(strstr(input,"HOST DOWNTIME STATE:")){
+
+				/* get host name */
+				temp_buffer=my_strtok(NULL,":");
+				temp_buffer=my_strtok(NULL,";");
+				strncpy(entry_host_name,(temp_buffer==NULL)?"":temp_buffer+1,sizeof(entry_host_name));
+				entry_host_name[sizeof(entry_host_name)-1]='\x0';
+
+				/* this host downtime entry must be added to all service subjects associated with the host! */
+				for(temp_subject=subject_list;temp_subject!=NULL;temp_subject=temp_subject->next){
+					
+					if(temp_subject->type!=SERVICE_SUBJECT)
+						continue;
+
+					if(strcmp(temp_subject->host_name,entry_host_name))
+						continue;
+
+					if(show_scheduled_downtime==FALSE)
+						continue;
+			
+					if(strstr(input,";STARTED;")) {
+						if (isLastLog && time_stamp == first_stamp) {
+							add_scheduled_downtime(AS_HOST_DOWNTIME_START,time_stamp,temp_subject);
+							}
+						}
+				        }
+			        }
 		        }
 		
 	        }
 
 	/* free memory and close the file */
-	free(input);
-	free(input2);
+	if (input) free(input);
+	if (input2) free(input2);
 	mmap_fclose(thefile);
 	
 	return;
@@ -3442,9 +3769,15 @@
 
 	/* display data for all hostgroups */
 	else{
-		for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next)
-			display_specific_hostgroup_availability(temp_hostgroup);
-	        }
+		if (show_hostgroups_summary==FALSE){
+			for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next){
+				display_specific_hostgroup_availability(temp_hostgroup);
+				}
+			}
+		else{
+			display_hostgroup_availability_summary();
+			}
+		}
 
 	return;
         }
@@ -3574,29 +3907,69 @@
         }
 
 
-/* display servicegroup availability */
-void display_servicegroup_availability(void){
-	servicegroup *temp_servicegroup;
+/* display availability summary for all hostgroups */
+void display_hostgroup_availability_summary(void){
+	hostgroup *temp_hostgroup;
+	GROUP_SUMMARY_TOTALS allGTs;
+	unsigned totalGroups = 0;
+	static char *bgclass[] = {"Even","Odd"};
+	int odd=1;
 
-	/* display data for a specific servicegroup */
-	if(show_all_servicegroups==FALSE){
-		temp_servicegroup=find_servicegroup(servicegroup_name);
-		display_specific_servicegroup_availability(temp_servicegroup);
-	        }
+	if(output_format==HTML_OUTPUT){
+		printf("<BR>\n");
+		printf("<DIV ALIGN=CENTER CLASS='dataTitle'>Availability Summary by Host Group:</DIV>\n");
+		printf("<BR>\n");
 
-	/* display data for all servicegroups */
-	else{
-		for(temp_servicegroup=servicegroup_list;temp_servicegroup!=NULL;temp_servicegroup=temp_servicegroup->next)
-			display_specific_servicegroup_availability(temp_servicegroup);
-	        }
+		printf("<DIV ALIGN=CENTER>\n");
+		printf("<TABLE BORDER=0 CLASS='data'>\n");
+		printf("<TR><TH CLASS='data'>Host Group</TH><TH CLASS='data'>%% Time Up</TH><TH CLASS='data'>%% Time Down</TH><TH CLASS='data'>%% Time Unreachable</TH><TH CLASS='data'>%% Time Undetermined</TH></TR>\n");
+		}
+	else if(output_format==CSV_OUTPUT){
+		printf("HOSTGROUP,UNSCHEDULED_UP,SCHEDULED_UP,UNSCHEDULED_DOWN,SCHEDULED_DOWN,UNSCHEDULED_UNREACHABLE,SCHEDULED_UNREACHABLE,INDETERMINATE\n");
+		}
 
-	return;
-        }
+	memset(&allGTs, 0, sizeof(allGTs));	/* Clear the Group Totals accumulator */
+
+	for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next){
+		if (display_specific_hostgroup_availability_summary(temp_hostgroup, bgclass[odd], &allGTs)) {
+			totalGroups++;
+			odd ^= 1;
+			}
+		}
 
+	/* Convert the running totals into averages */
+	allGTs.average_percent_time_up /= totalGroups;
+	allGTs.average_percent_time_up_known /= totalGroups;
+	allGTs.average_percent_time_down /= totalGroups;
+	allGTs.average_percent_time_down_known /= totalGroups;
+	allGTs.average_percent_time_unreachable /= totalGroups;
+	allGTs.average_percent_time_unreachable_known /= totalGroups;
+	allGTs.average_percent_time_indeterminate /= totalGroups;
+
+	/* Display the averages-of-the-averages row */
+	if(output_format==HTML_OUTPUT){
+		printf("<tr CLASS='data%s'><td CLASS='data'><center><b>",bgclass[odd]);
+		printf("Average (%u Groups)", totalGroups);
+		printf("</b></center></td><td CLASS='hostUP'>%2.3f%% (%2.3f%%)</td><td CLASS='hostDOWN'>%2.3f%% (%2.3f%%)</td><td CLASS='hostUNREACHABLE'>%2.3f%% (%2.3f%%)</td><td class='data%s'>%2.3f%%</td></tr>",
+			allGTs.average_percent_time_up,
+			allGTs.average_percent_time_up_known,
+			allGTs.average_percent_time_down,
+			allGTs.average_percent_time_down_known,
+			allGTs.average_percent_time_unreachable,
+			allGTs.average_percent_time_unreachable_known,
+			bgclass[odd],
+			allGTs.average_percent_time_indeterminate);
 
+		printf("</table>\n");
+		printf("</DIV>\n");
+		}
 
-/* display availability for a specific servicegroup */
-void display_specific_servicegroup_availability(servicegroup *sg){
+	return;
+	}
+
+
+/* display availability summary for a specific hostgroup */
+int display_specific_hostgroup_availability_summary(hostgroup *hg, char *bgclass, GROUP_SUMMARY_TOTALS *pgt){
 	unsigned long total_time;
 	unsigned long time_determinate;
 	unsigned long time_indeterminate;
@@ -3608,14 +3981,6 @@
 	double percent_time_down_known=0.0;
 	double percent_time_unreachable_known=0.0;
 	double percent_time_indeterminate=0.0;
-	double percent_time_ok=0.0;
-	double percent_time_warning=0.0;
-	double percent_time_unknown=0.0;
-	double percent_time_critical=0.0;
-	double percent_time_ok_known=0.0;
-	double percent_time_warning_known=0.0;
-	double percent_time_unknown_known=0.0;
-	double percent_time_critical_known=0.0;
 
 	double average_percent_time_up=0.0;
 	double average_percent_time_up_known=0.0;
@@ -3623,41 +3988,22 @@
 	double average_percent_time_down_known=0.0;
 	double average_percent_time_unreachable=0.0;
 	double average_percent_time_unreachable_known=0.0;
-	double average_percent_time_ok=0.0;
-	double average_percent_time_ok_known=0.0;
-	double average_percent_time_unknown=0.0;
-	double average_percent_time_unknown_known=0.0;
-	double average_percent_time_warning=0.0;
-	double average_percent_time_warning_known=0.0;
-	double average_percent_time_critical=0.0;
-	double average_percent_time_critical_known=0.0;
 	double average_percent_time_indeterminate=0.0;
 
 	int current_subject=0;
 
-	char *bgclass="";
-	int odd=1;
 	host *temp_host;
-	service *temp_service;
-	char last_host[MAX_INPUT_BUFFER];
 
-	if(sg==NULL)
-		return;
+	if(hg==NULL)
+		return 0;
 
-	/* the user isn't authorized to view this servicegroup */
-	if(is_authorized_for_servicegroup(sg,&current_authdata)==FALSE)
-		return;
+	/* the user isn't authorized to view this hostgroup */
+	if(is_authorized_for_hostgroup(hg,&current_authdata)==FALSE)
+		return 0;
 
 	/* calculate total time during period based on timeperiod used for reporting */
 	total_time=calculate_total_time(t1,t2);
 
-	printf("<BR><BR>\n");
-	printf("<DIV ALIGN=CENTER CLASS='dataTitle'>Servicegroup '%s' Host State Breakdowns:</DIV>\n",sg->group_name);
-
-	printf("<DIV ALIGN=CENTER>\n");
-	printf("<TABLE BORDER=0 CLASS='data'>\n");
-	printf("<TR><TH CLASS='data'>Host</TH><TH CLASS='data'>%% Time Up</TH><TH CLASS='data'>%% Time Down</TH><TH CLASS='data'>%% Time Unreachable</TH><TH CLASS='data'>%% Time Undetermined</TH></TR>\n");
-
 	for(temp_subject=subject_list;temp_subject!=NULL;temp_subject=temp_subject->next){
 
 		if(temp_subject->type!=HOST_SUBJECT)
@@ -3667,7 +4013,7 @@
 		if(temp_host==NULL)
 			continue;
 
-		if(is_host_member_of_servicegroup(sg,temp_host)==FALSE)
+		if(is_host_member_of_hostgroup(hg,temp_host)==FALSE)
 			continue;
 
 		current_subject++;
@@ -3696,19 +4042,6 @@
 	                        }
 		        }
 
-		if(odd){
-			odd=0;
-			bgclass="Odd";
-	                }
-		else{
-			odd=1;
-			bgclass="Even";
-	                }
-
-		printf("<tr CLASS='data%s'><td CLASS='data%s'>",bgclass,bgclass);
-		host_report_url(temp_subject->host_name,temp_subject->host_name);
-		printf("</td><td CLASS='hostUP'>%2.3f%% (%2.3f%%)</td><td CLASS='hostDOWN'>%2.3f%% (%2.3f%%)</td><td CLASS='hostUNREACHABLE'>%2.3f%% (%2.3f%%)</td><td class='data%s'>%2.3f%%</td></tr>\n",percent_time_up,percent_time_up_known,percent_time_down,percent_time_down_known,percent_time_unreachable,percent_time_unreachable_known,bgclass,percent_time_indeterminate);
-
 		get_running_average(&average_percent_time_up,percent_time_up,current_subject);
 		get_running_average(&average_percent_time_up_known,percent_time_up_known,current_subject);
 		get_running_average(&average_percent_time_down,percent_time_down,current_subject);
@@ -3719,39 +4052,761 @@
                 }
 
 	/* average statistics */
-	if(odd){
-		odd=0;
-		bgclass="Odd";
-	        }
-	else{
-		odd=1;
-		bgclass="Even";
-      	        }
-	printf("<tr CLASS='data%s'><td CLASS='data%s'>Average</td><td CLASS='hostUP'>%2.3f%% (%2.3f%%)</td><td CLASS='hostDOWN'>%2.3f%% (%2.3f%%)</td><td CLASS='hostUNREACHABLE'>%2.3f%% (%2.3f%%)</td><td class='data%s'>%2.3f%%</td></tr>",bgclass,bgclass,average_percent_time_up,average_percent_time_up_known,average_percent_time_down,average_percent_time_down_known,average_percent_time_unreachable,average_percent_time_unreachable_known,bgclass,average_percent_time_indeterminate);
-
-	printf("</table>\n");
-	printf("</DIV>\n");
+	if(output_format==HTML_OUTPUT){
+		printf("<tr CLASS='data%s'><td CLASS='data%s'>%s</td><td CLASS='hostUP'>%2.3f%% (%2.3f%%)</td><td CLASS='hostDOWN'>%2.3f%% (%2.3f%%)</td><td CLASS='hostUNREACHABLE'>%2.3f%% (%2.3f%%)</td><td class='data%s'>%2.3f%%</td></tr>",bgclass,bgclass,hg->group_name,average_percent_time_up,average_percent_time_up_known,average_percent_time_down,average_percent_time_down_known,average_percent_time_unreachable,average_percent_time_unreachable_known,bgclass,average_percent_time_indeterminate);
+		}
+	else if(output_format==CSV_OUTPUT){
+		printf("%s,%2.3f,%2.3f,%2.3f,%2.3f,%2.3f,%2.3f,%2.3f\n",hg->group_name,average_percent_time_up,average_percent_time_up_known,average_percent_time_down,average_percent_time_down_known,average_percent_time_unreachable,average_percent_time_unreachable_known,average_percent_time_indeterminate);
+		}
 
-	printf("<BR>\n");
-	printf("<DIV ALIGN=CENTER CLASS='dataTitle'>Servicegroup '%s' Service State Breakdowns:</DIV>\n",sg->group_name);
 
-	printf("<DIV ALIGN=CENTER>\n");
-	printf("<TABLE BORDER=0 CLASS='data'>\n");
-	printf("<TR><TH CLASS='data'>Host</TH><TH CLASS='data'>Service</TH><TH CLASS='data'>%% Time OK</TH><TH CLASS='data'>%% Time Warning</TH><TH CLASS='data'>%% Time Unknown</TH><TH CLASS='data'>%% Time Critical</TH><TH CLASS='data'>%% Time Undetermined</TH></TR>\n");
+	/* Add this group's averages to the running totals */
+	pgt->average_percent_time_up += average_percent_time_up;
+	pgt->average_percent_time_up_known += average_percent_time_up_known;
+	pgt->average_percent_time_down += average_percent_time_down;
+	pgt->average_percent_time_down_known += average_percent_time_down_known;
+	pgt->average_percent_time_unreachable += average_percent_time_unreachable;
+	pgt->average_percent_time_unreachable_known += average_percent_time_unreachable_known;
+	pgt->average_percent_time_indeterminate += average_percent_time_indeterminate;
 
-	current_subject=0;
-	average_percent_time_indeterminate=0.0;
+	return 1;
+	}
 
-	for(temp_subject=subject_list;temp_subject!=NULL;temp_subject=temp_subject->next){
 
-		if(temp_subject->type!=SERVICE_SUBJECT)
-			continue;
+/* display servicegroup availability */
+void display_servicegroup_availability(void){
+	servicegroup *temp_servicegroup;
 
-		temp_service=find_service(temp_subject->host_name,temp_subject->service_description);
-		if(temp_service==NULL)
-			continue;
+	/* display data for a specific servicegroup */
+	if(show_all_servicegroups==FALSE){
+		temp_servicegroup=find_servicegroup(servicegroup_name);
+		display_specific_servicegroup_availability(temp_servicegroup);
+	        }
 
-		if(is_service_member_of_servicegroup(sg,temp_service)==FALSE)
+	/* display data for all servicegroups */
+	else{
+		if (show_servicegroups_summary==FALSE){
+			for(temp_servicegroup=servicegroup_list;temp_servicegroup!=NULL;temp_servicegroup=temp_servicegroup->next){
+				display_specific_servicegroup_availability(temp_servicegroup);
+				}
+			}
+		else{
+			display_servicegroup_availability_summary();
+			}
+		}
+
+	return;
+        }
+
+
+
+/* display availability for a specific servicegroup */
+void display_specific_servicegroup_availability(servicegroup *sg){
+	unsigned long total_time;
+	unsigned long time_determinate;
+	unsigned long time_indeterminate;
+	avail_subject *temp_subject;
+	double percent_time_up=0.0;
+	double percent_time_down=0.0;
+	double percent_time_unreachable=0.0;
+	double percent_time_up_known=0.0;
+	double percent_time_down_known=0.0;
+	double percent_time_unreachable_known=0.0;
+	double percent_time_indeterminate=0.0;
+	double percent_time_ok=0.0;
+	double percent_time_warning=0.0;
+	double percent_time_unknown=0.0;
+	double percent_time_critical=0.0;
+	double percent_time_ok_known=0.0;
+	double percent_time_warning_known=0.0;
+	double percent_time_unknown_known=0.0;
+	double percent_time_critical_known=0.0;
+
+	double percent_time_critical_scheduled=0.0;
+	double percent_time_critical_unscheduled=0.0;
+	double percent_time_critical_scheduled_known=0.0;
+	double percent_time_critical_unscheduled_known=0.0;
+
+	double percent_time_unknown_scheduled=0.0;
+	double percent_time_unknown_unscheduled=0.0;
+	double percent_time_unknown_scheduled_known=0.0;
+	double percent_time_unknown_unscheduled_known=0.0;
+
+	double percent_time_warning_scheduled=0.0;
+	double percent_time_warning_unscheduled=0.0;
+	double percent_time_warning_scheduled_known=0.0;
+	double percent_time_warning_unscheduled_known=0.0;
+
+	double percent_time_ok_scheduled=0.0;
+	double percent_time_ok_unscheduled=0.0;
+	double percent_time_ok_scheduled_known=0.0;
+	double percent_time_ok_unscheduled_known=0.0;
+
+	double average_percent_time_up=0.0;
+	double average_percent_time_up_known=0.0;
+	double average_percent_time_down=0.0;
+	double average_percent_time_down_known=0.0;
+	double average_percent_time_unreachable=0.0;
+	double average_percent_time_unreachable_known=0.0;
+	double average_percent_time_ok=0.0;
+	double average_percent_time_ok_known=0.0;
+	double average_percent_time_unknown=0.0;
+	double average_percent_time_unknown_known=0.0;
+	double average_percent_time_warning=0.0;
+	double average_percent_time_warning_known=0.0;
+	double average_percent_time_critical=0.0;
+	double average_percent_time_critical_known=0.0;
+	double average_percent_time_indeterminate=0.0;
+
+	double percent_time_up_scheduled=0.0;
+	double percent_time_up_unscheduled=0.0;
+	double percent_time_down_scheduled=0.0;
+	double percent_time_down_unscheduled=0.0;
+	double percent_time_unreachable_scheduled=0.0;
+	double percent_time_unreachable_unscheduled=0.0;
+	double percent_time_up_scheduled_known=0.0;
+	double percent_time_up_unscheduled_known=0.0;
+	double percent_time_down_scheduled_known=0.0;
+	double percent_time_down_unscheduled_known=0.0;
+	double percent_time_unreachable_scheduled_known=0.0;
+	double percent_time_unreachable_unscheduled_known=0.0;
+
+	int current_subject=0;
+
+	double percent_time_indeterminate_scheduled=0.0;
+	double percent_time_indeterminate_unscheduled=0.0;
+	double percent_time_indeterminate_notrunning=0.0;
+	double percent_time_indeterminate_nodata=0.0;
+
+	char *bgclass="";
+	int odd=1;
+	host *temp_host;
+	service *temp_service;
+	char last_host[MAX_INPUT_BUFFER];
+
+	if(sg==NULL)
+		return;
+
+	/* the user isn't authorized to view this servicegroup */
+	if(is_authorized_for_servicegroup(sg,&current_authdata)==FALSE)
+		return;
+
+	/* calculate total time during period based on timeperiod used for reporting */
+	total_time=calculate_total_time(t1,t2);
+
+	printf("<BR><BR>\n");
+	printf("<DIV ALIGN=CENTER CLASS='dataTitle'>Servicegroup '%s' Host State Breakdowns:</DIV>\n",sg->group_name);
+
+	printf("<DIV ALIGN=CENTER>\n");
+	printf("<TABLE BORDER=0 CLASS='data'>\n");
+	printf("<TR><TH CLASS='data'>Host</TH><TH CLASS='data'>%% Time Up</TH><TH CLASS='data'>%% Time Down</TH><TH CLASS='data'>%% Time Unreachable</TH><TH CLASS='data'>%% Time Undetermined</TH></TR>\n");
+
+	for(temp_subject=subject_list;temp_subject!=NULL;temp_subject=temp_subject->next){
+
+		if(temp_subject->type!=HOST_SUBJECT)
+			continue;
+
+		temp_host=find_host(temp_subject->host_name);
+		if(temp_host==NULL)
+			continue;
+
+		if(is_host_member_of_servicegroup(sg,temp_host)==FALSE)
+			continue;
+
+		current_subject++;
+
+		/* reset variables */
+		percent_time_up=0.0;
+		percent_time_up_scheduled=0.0;
+		percent_time_up_unscheduled=0.0;
+		percent_time_down=0.0;
+		percent_time_down_scheduled=0.0;
+		percent_time_down_unscheduled=0.0;
+		percent_time_unreachable=0.0;
+		percent_time_unreachable_scheduled=0.0;
+		percent_time_unreachable_unscheduled=0.0;
+		percent_time_indeterminate=0.0;
+		percent_time_indeterminate_scheduled=0.0;
+		percent_time_indeterminate_unscheduled=0.0;
+		percent_time_indeterminate_notrunning=0.0;
+		percent_time_indeterminate_nodata=0.0;
+		percent_time_up_known=0.0;
+		percent_time_up_scheduled_known=0.0;
+		percent_time_up_unscheduled_known=0.0;
+		percent_time_down_known=0.0;
+		percent_time_down_scheduled_known=0.0;
+		percent_time_down_unscheduled_known=0.0;
+		percent_time_unreachable_known=0.0;
+		percent_time_unreachable_scheduled_known=0.0;
+		percent_time_unreachable_unscheduled_known=0.0;
+
+		time_determinate=temp_subject->time_up+temp_subject->time_down+temp_subject->time_unreachable;
+		time_indeterminate=total_time-time_determinate;
+	
+		if(total_time>0){
+			percent_time_up=(double)(((double)temp_subject->time_up*100.0)/(double)total_time);
+			percent_time_up_scheduled=(double)(((double)temp_subject->scheduled_time_up*100.0)/(double)total_time);
+			percent_time_up_unscheduled=percent_time_up-percent_time_up_scheduled;
+			percent_time_down=(double)(((double)temp_subject->time_down*100.0)/(double)total_time);
+			percent_time_down_scheduled=(double)(((double)temp_subject->scheduled_time_down*100.0)/(double)total_time);
+			percent_time_down_unscheduled=percent_time_down-percent_time_down_scheduled;
+			percent_time_unreachable=(double)(((double)temp_subject->time_unreachable*100.0)/(double)total_time);
+			percent_time_unreachable_scheduled=(double)(((double)temp_subject->scheduled_time_unreachable*100.0)/(double)total_time);
+			percent_time_unreachable_unscheduled=percent_time_unreachable-percent_time_unreachable_scheduled;
+			percent_time_indeterminate=(double)(((double)time_indeterminate*100.0)/(double)total_time);
+			percent_time_indeterminate_scheduled=(double)(((double)temp_subject->scheduled_time_indeterminate*100.0)/(double)total_time);
+			percent_time_indeterminate_unscheduled=percent_time_indeterminate-percent_time_indeterminate_scheduled;
+			percent_time_indeterminate_notrunning=(double)(((double)temp_subject->time_indeterminate_notrunning*100.0)/(double)total_time);
+			percent_time_indeterminate_nodata=(double)(((double)temp_subject->time_indeterminate_nodata*100.0)/(double)total_time);
+			if(time_determinate>0){
+				percent_time_up_known=(double)(((double)temp_subject->time_up*100.0)/(double)time_determinate);
+				percent_time_up_scheduled_known=(double)(((double)temp_subject->scheduled_time_up*100.0)/(double)time_determinate);
+				percent_time_up_unscheduled_known=percent_time_up_known-percent_time_up_scheduled_known;
+				percent_time_down_known=(double)(((double)temp_subject->time_down*100.0)/(double)time_determinate);
+				percent_time_down_scheduled_known=(double)(((double)temp_subject->scheduled_time_down*100.0)/(double)time_determinate);
+				percent_time_down_unscheduled_known=percent_time_down_known-percent_time_down_scheduled_known;
+				percent_time_unreachable_known=(double)(((double)temp_subject->time_unreachable*100.0)/(double)time_determinate);
+				percent_time_unreachable_scheduled_known=(double)(((double)temp_subject->scheduled_time_unreachable*100.0)/(double)time_determinate);
+				percent_time_unreachable_unscheduled_known=percent_time_unreachable_known-percent_time_unreachable_scheduled_known;
+	                        }
+                        }
+
+		if(odd){
+			odd=0;
+			bgclass="Odd";
+	                }
+		else{
+			odd=1;
+			bgclass="Even";
+	                }
+
+		printf("<tr CLASS='data%s'><td CLASS='data%s'>",bgclass,bgclass);
+		host_report_url(temp_subject->host_name,temp_subject->host_name);
+		printf("</td><td CLASS='hostUP'>%2.3f%% (%2.3f%%)</td><td CLASS='hostDOWN'>%2.3f%% (%2.3f%%)</td><td CLASS='hostUNREACHABLE'>%2.3f%% (%2.3f%%)</td><td class='data%s'>%2.3f%%</td></tr>\n",percent_time_up_unscheduled_known,percent_time_up_scheduled_known,percent_time_down_unscheduled_known,percent_time_down_scheduled_known,percent_time_unreachable_unscheduled_known,percent_time_unreachable_scheduled_known,bgclass,percent_time_indeterminate);
+
+		get_running_average(&average_percent_time_up,percent_time_up_unscheduled_known,current_subject);
+		get_running_average(&average_percent_time_up_known,percent_time_up_scheduled_known,current_subject);
+		get_running_average(&average_percent_time_down,percent_time_down_unscheduled_known,current_subject);
+		get_running_average(&average_percent_time_down_known,percent_time_down_scheduled_known,current_subject);
+		get_running_average(&average_percent_time_unreachable,percent_time_unreachable_unscheduled_known,current_subject);
+		get_running_average(&average_percent_time_unreachable_known,percent_time_unreachable_scheduled_known,current_subject);
+		get_running_average(&average_percent_time_indeterminate,percent_time_indeterminate,current_subject);
+                }
+
+	/* average statistics */
+	if(odd){
+		odd=0;
+		bgclass="Odd";
+	        }
+	else{
+		odd=1;
+		bgclass="Even";
+      	        }
+	printf("<tr CLASS='data%s'><td CLASS='data%s'>Average</td><td CLASS='hostUP'>%2.3f%% (%2.3f%%)</td><td CLASS='hostDOWN'>%2.3f%% (%2.3f%%)</td><td CLASS='hostUNREACHABLE'>%2.3f%% (%2.3f%%)</td><td class='data%s'>%2.3f%%</td></tr>",bgclass,bgclass,average_percent_time_up,average_percent_time_up_known,average_percent_time_down,average_percent_time_down_known,average_percent_time_unreachable,average_percent_time_unreachable_known,bgclass,average_percent_time_indeterminate);
+
+	printf("</table>\n");
+	printf("</DIV>\n");
+
+	printf("<BR>\n");
+	printf("<DIV ALIGN=CENTER CLASS='dataTitle'>Servicegroup '%s' Service State Breakdowns:</DIV>\n",sg->group_name);
+
+	printf("<DIV ALIGN=CENTER>\n");
+	printf("<TABLE BORDER=0 CLASS='data'>\n");
+	printf("<TR><TH CLASS='data'>Host</TH><TH CLASS='data'>Service</TH><TH CLASS='data'>%% Time OK</TH><TH CLASS='data'>%% Time Warning</TH><TH CLASS='data'>%% Time Unknown</TH><TH CLASS='data'>%% Time Critical</TH><TH CLASS='data'>%% Time Undetermined</TH></TR>\n");
+
+	current_subject=0;
+	average_percent_time_indeterminate=0.0;
+
+	for(temp_subject=subject_list;temp_subject!=NULL;temp_subject=temp_subject->next){
+
+		if(temp_subject->type!=SERVICE_SUBJECT)
+			continue;
+
+		temp_service=find_service(temp_subject->host_name,temp_subject->service_description);
+		if(temp_service==NULL)
+			continue;
+
+		if(is_service_member_of_servicegroup(sg,temp_service)==FALSE)
+			continue;
+
+		current_subject++;
+
+		time_determinate=temp_subject->time_ok+temp_subject->time_warning+temp_subject->time_unknown+temp_subject->time_critical;
+		time_indeterminate=total_time-time_determinate;
+
+		/* adjust indeterminate time due to insufficient data (not all was caught) */
+		temp_subject->time_indeterminate_nodata=time_indeterminate-temp_subject->time_indeterminate_notrunning;
+
+		/* initialize values */
+		percent_time_ok=0.0;
+		percent_time_ok_scheduled=0.0;
+		percent_time_ok_unscheduled=0.0;
+		percent_time_warning=0.0;
+		percent_time_warning_scheduled=0.0;
+		percent_time_warning_unscheduled=0.0;
+		percent_time_unknown=0.0;
+		percent_time_unknown_scheduled=0.0;
+		percent_time_unknown_unscheduled=0.0;
+		percent_time_critical=0.0;
+		percent_time_critical_scheduled=0.0;
+		percent_time_critical_unscheduled=0.0;
+		percent_time_indeterminate=0.0;
+		percent_time_indeterminate_scheduled=0.0;
+		percent_time_indeterminate_unscheduled=0.0;
+		percent_time_indeterminate_notrunning=0.0;
+		percent_time_indeterminate_nodata=0.0;
+		percent_time_ok_known=0.0;
+		percent_time_ok_scheduled_known=0.0;
+		percent_time_ok_unscheduled_known=0.0;
+		percent_time_warning_known=0.0;
+		percent_time_warning_scheduled_known=0.0;
+		percent_time_warning_unscheduled_known=0.0;
+		percent_time_unknown_known=0.0;
+		percent_time_unknown_scheduled_known=0.0;
+		percent_time_unknown_unscheduled_known=0.0;
+		percent_time_critical_known=0.0;
+		percent_time_critical_scheduled_known=0.0;
+		percent_time_critical_unscheduled_known=0.0;
+
+		if(total_time>0){
+			percent_time_ok=(double)(((double)temp_subject->time_ok*100.0)/(double)total_time);
+			percent_time_ok_scheduled=(double)(((double)temp_subject->scheduled_time_ok*100.0)/(double)total_time);
+			percent_time_ok_unscheduled=percent_time_ok-percent_time_ok_scheduled;
+			percent_time_warning=(double)(((double)temp_subject->time_warning*100.0)/(double)total_time);
+			percent_time_warning_scheduled=(double)(((double)temp_subject->scheduled_time_unknown*100.0)/(double)total_time);
+			percent_time_warning_unscheduled=percent_time_warning-percent_time_warning_scheduled;
+			percent_time_unknown=(double)(((double)temp_subject->time_unknown*100.0)/(double)total_time);
+			percent_time_unknown_scheduled=(double)(((double)temp_subject->scheduled_time_unknown*100.0)/(double)total_time);
+			percent_time_unknown_unscheduled=percent_time_unknown-percent_time_unknown_scheduled;
+			percent_time_critical=(double)(((double)temp_subject->time_critical*100.0)/(double)total_time);
+			percent_time_critical_scheduled=(double)(((double)temp_subject->scheduled_time_critical*100.0)/(double)total_time);
+			percent_time_critical_unscheduled=percent_time_critical-percent_time_critical_scheduled;
+			percent_time_indeterminate=(double)(((double)time_indeterminate*100.0)/(double)total_time);
+			percent_time_indeterminate_scheduled=(double)(((double)temp_subject->scheduled_time_indeterminate*100.0)/(double)total_time);
+			percent_time_indeterminate_unscheduled=percent_time_indeterminate-percent_time_indeterminate_scheduled;
+			percent_time_indeterminate_notrunning=(double)(((double)temp_subject->time_indeterminate_notrunning*100.0)/(double)total_time);
+			percent_time_indeterminate_nodata=(double)(((double)temp_subject->time_indeterminate_nodata*100.0)/(double)total_time);
+			if(time_determinate>0){
+				percent_time_ok_known=(double)(((double)temp_subject->time_ok*100.0)/(double)time_determinate);
+				percent_time_ok_scheduled_known=(double)(((double)temp_subject->scheduled_time_ok*100.0)/(double)time_determinate);
+				percent_time_ok_unscheduled_known=percent_time_ok_known-percent_time_ok_scheduled_known;
+				percent_time_warning_known=(double)(((double)temp_subject->time_warning*100.0)/(double)time_determinate);
+				percent_time_warning_scheduled_known=(double)(((double)temp_subject->scheduled_time_warning*100.0)/(double)time_determinate);
+				percent_time_warning_unscheduled_known=percent_time_warning_known-percent_time_warning_scheduled_known;
+				percent_time_unknown_known=(double)(((double)temp_subject->time_unknown*100.0)/(double)time_determinate);
+				percent_time_unknown_scheduled_known=(double)(((double)temp_subject->scheduled_time_unknown*100.0)/(double)time_determinate);
+				percent_time_unknown_unscheduled_known=percent_time_unknown_known-percent_time_unknown_scheduled_known;
+				percent_time_critical_known=(double)(((double)temp_subject->time_critical*100.0)/(double)time_determinate);
+				percent_time_critical_scheduled_known=(double)(((double)temp_subject->scheduled_time_critical*100.0)/(double)time_determinate);
+				percent_time_critical_unscheduled_known=percent_time_critical_known-percent_time_critical_scheduled_known;
+	                        }
+                        }
+
+		if(odd){
+			odd=0;
+			bgclass="Odd";
+	                }
+		else{
+			odd=1;
+			bgclass="Even";
+	                }
+
+		printf("<tr CLASS='data%s'><td CLASS='data%s'>",bgclass,bgclass);
+		if(strcmp(temp_subject->host_name,last_host))
+			host_report_url(temp_subject->host_name,temp_subject->host_name);
+		printf("</td><td CLASS='data%s'>",bgclass);
+		service_report_url(temp_subject->host_name,temp_subject->service_description,temp_subject->service_description);
+		printf("</td><td CLASS='serviceOK'>%2.3f%% (%2.3f%%)</td><td CLASS='serviceWARNING'>%2.3f%% (%2.3f%%)</td><td CLASS='serviceUNKNOWN'>%2.3f%% (%2.3f%%)</td><td class='serviceCRITICAL'>%2.3f%% (%2.3f%%)</td><td class='data%s'>%2.3f%%</td></tr>\n",percent_time_ok_unscheduled_known,percent_time_ok_scheduled_known,percent_time_warning_unscheduled_known,percent_time_warning_scheduled_known,percent_time_unknown_unscheduled_known,percent_time_unknown_scheduled_known,percent_time_critical_unscheduled_known,percent_time_critical_scheduled_known,bgclass,percent_time_indeterminate);
+
+		strncpy(last_host,temp_subject->host_name,sizeof(last_host)-1);
+		last_host[sizeof(last_host)-1]='\x0';
+
+		get_running_average(&average_percent_time_ok,percent_time_ok_unscheduled_known,current_subject);
+		get_running_average(&average_percent_time_ok_known,percent_time_ok_scheduled_known,current_subject);
+		get_running_average(&average_percent_time_unknown,percent_time_unknown_unscheduled_known,current_subject);
+		get_running_average(&average_percent_time_unknown_known,percent_time_unknown_scheduled_known,current_subject);
+		get_running_average(&average_percent_time_warning,percent_time_warning_unscheduled_known,current_subject);
+		get_running_average(&average_percent_time_warning_known,percent_time_warning_scheduled_known,current_subject);
+		get_running_average(&average_percent_time_critical,percent_time_critical_unscheduled_known,current_subject);
+		get_running_average(&average_percent_time_critical_known,percent_time_critical_scheduled_known,current_subject);
+		get_running_average(&average_percent_time_indeterminate,percent_time_indeterminate,current_subject);
+	        }
+
+	/* display average stats */
+	if(odd){
+		odd=0;
+		bgclass="Odd";
+	        }
+	else{
+		odd=1;
+		bgclass="Even";
+	        }
+
+	printf("<tr CLASS='data%s'><td CLASS='data%s' colspan='2'>Average</td><td CLASS='serviceOK'>%2.3f%% (%2.3f%%)</td><td CLASS='serviceWARNING'>%2.3f%% (%2.3f%%)</td><td CLASS='serviceUNKNOWN'>%2.3f%% (%2.3f%%)</td><td class='serviceCRITICAL'>%2.3f%% (%2.3f%%)</td><td class='data%s'>%2.3f%%</td></tr>\n",bgclass,bgclass,average_percent_time_ok,average_percent_time_ok_known,average_percent_time_warning,average_percent_time_warning_known,average_percent_time_unknown,average_percent_time_unknown_known,average_percent_time_critical,average_percent_time_critical_known,bgclass,average_percent_time_indeterminate);
+
+	printf("</table>\n");
+	printf("</DIV>\n");
+
+	return;
+        }
+
+/* Service group sorting comparator */
+int sg_compare (servicegroup_stats *sg_left, servicegroup_stats *sg_right)
+{
+	int result;
+#ifndef SIMPLE_SLAA_SORT
+	int leftWithinSLA, rightWithinSLA;
+#endif
+
+	/* Perform an ascending sort comparison */
+	switch (sort_column)
+	{
+		case SG_SORT_NAME:
+			result = strcasecmp(sg_left->sg->group_name, sg_right->sg->group_name);
+			break;
+		case SG_SORT_OK:
+			result = (int)(sg_left->average_percent_time_ok*1000.0+0.5) - (int)(sg_right->average_percent_time_ok*1000.0+0.5);
+			break;
+		case SG_SORT_WARN:
+			result = (int)(sg_left->average_percent_time_warning*1000.0+0.5) - (int)(sg_right->average_percent_time_warning*1000.0+0.5);
+			break;
+		case SG_SORT_UNK:
+			result = (int)(sg_left->average_percent_time_unknown*1000.0+0.5) - (int)(sg_right->average_percent_time_unknown*1000.0+0.5);
+			break;
+		case SG_SORT_CRIT:
+			result = (int)(sg_left->average_percent_time_critical*1000.0+0.5) - (int)(sg_right->average_percent_time_critical*1000.0+0.5);
+			break;
+		case SG_SORT_UND:
+			result = (int)(sg_left->average_percent_time_indeterminate*1000.0+0.5) - (int)(sg_right->average_percent_time_indeterminate*1000.0+0.5);
+			break;
+		case SG_SORT_SLAA:
+#ifndef SIMPLE_SLAA_SORT
+			leftWithinSLA  = (((int)(sg_left->sla_actual*1000.0+0.5) < (int)(sg_left->sg->sla_target*1000.0+0.5)) ? 0 : 1);
+			rightWithinSLA = (((int)(sg_right->sla_actual*1000.0+0.5) < (int)(sg_right->sg->sla_target*1000.0+0.5)) ? 0 : 1);
+			if ((result = leftWithinSLA - rightWithinSLA) == 0)
+#endif
+				result = (int)(sg_left->sla_actual*1000.0+0.5) - (int)(sg_right->sla_actual*1000.0+0.5);
+			break;
+		case SG_SORT_SLAT:
+			result = (int)(sg_left->sg->sla_target*1000.0+0.5) - (int)(sg_right->sg->sla_target*1000.0+0.5);
+			break;
+		default:
+			result = 0;
+	}
+
+	/* Reverse the result if performing a descending sort */
+	if (sort_direction == SORT_DIR_DESC)
+		result *= (-1);
+
+	return result;
+}
+
+/* display availability summary for all servicegroups */
+void display_servicegroup_availability_summary(void){
+	servicegroup *temp_servicegroup;
+	GROUP_SUMMARY_TOTALS allGTs;
+	unsigned totalGroups = 0;
+	static char *bgclass[] = {"Even","Odd"};
+	static char *col_header[] = {"Service Group","% Time OK","% Time Warning","% Time Unknown","% Time Critical","% Time Undetermined","Actual SLA","Target SLA",NULL};
+	int i, odd=1;
+	int totalWithinSLA, totalOutOfSLA;
+	double percentWithinSLA, percentOutOfSLA;
+
+
+	memset(&allGTs, 0, sizeof(allGTs));	/* Clear the Group Totals accumulator */
+
+	/* Gather the "short" list of service groups and statistics */
+	for(temp_servicegroup=servicegroup_list;temp_servicegroup!=NULL;temp_servicegroup=temp_servicegroup->next){
+		if (display_specific_servicegroup_availability_summary(temp_servicegroup, &sg_stat_array[totalGroups])) {
+			/* Save the pointer to the selected service group */
+			sg_stat_array[totalGroups].sg = temp_servicegroup;
+
+			/* Add this service group's statistics to the Group Totals accumulator */
+			allGTs.average_percent_time_ok += sg_stat_array[totalGroups].average_percent_time_ok;
+			allGTs.average_percent_time_ok_known += sg_stat_array[totalGroups].average_percent_time_ok_known;
+			allGTs.average_percent_time_unknown += sg_stat_array[totalGroups].average_percent_time_unknown;
+			allGTs.average_percent_time_unknown_known += sg_stat_array[totalGroups].average_percent_time_unknown_known;
+			allGTs.average_percent_time_warning += sg_stat_array[totalGroups].average_percent_time_warning;
+			allGTs.average_percent_time_warning_known += sg_stat_array[totalGroups].average_percent_time_warning_known;
+			allGTs.average_percent_time_critical += sg_stat_array[totalGroups].average_percent_time_critical;
+			allGTs.average_percent_time_critical_known += sg_stat_array[totalGroups].average_percent_time_critical_known;
+			allGTs.average_percent_time_indeterminate += sg_stat_array[totalGroups].average_percent_time_indeterminate;
+			allGTs.sla_actual += sg_stat_array[totalGroups].sla_actual;
+			allGTs.sla_target += sg_stat_array[totalGroups].sla_target;
+
+			totalGroups++;	/* Increment the service group counter */
+			}
+		}
+
+	/* Sort the list */
+	qsort(sg_stat_array, totalGroups, sizeof(servicegroup_stats), (__compar_fn_t)sg_compare);
+
+	/* Display the table header */
+	if(output_format==HTML_OUTPUT){
+		printf("<BR>\n");
+		printf("<DIV ALIGN=CENTER CLASS='dataTitle'>Availability Summary by Service Group:</DIV>\n");
+		printf("<BR>\n");
+
+		printf("<DIV ALIGN=CENTER>\n");
+		printf("<TABLE BORDER=0 CLASS='data'>\n");
+		printf("<TR>");
+		for (i=0; col_header[i]; i++)
+			avail_sort_url(servicegroup_name, col_header[i], i);
+		printf("</TR>\n");
+		}
+	else if(output_format==CSV_OUTPUT){
+		printf("SERVICEGROUP,UNSCHEDULED_OK,SCHEDULED_OK,UNSCHEDULED_WARN,SCHEDULED_WARN,UNSCHEDULED_UNKNOWN,SCHEDULED_UNKNOWN,UNSCHEDULED_CRITICAL,SCHEDULED_CRITICAL,INDETERMINATE,SLA_ACTUAL,SCHEDULED_TOTAL,SLA_TARGET,IS_WITHIN_SLA\n");
+		}
+
+	/* Display the sorted list */
+	totalWithinSLA = 0;
+	for (i=0; i < totalGroups; i++) {
+		/* display average stats */
+		if(output_format==HTML_OUTPUT){
+			printf("<tr CLASS='data%s'><td CLASS='data%s'>",bgclass[odd],bgclass[odd]);
+			service_detail_url(sg_stat_array[i].sg->group_name, sg_stat_array[i].sg->group_name);
+			printf("</td><td CLASS='serviceOK'>%2.3f%% (%2.3f%%)</td><td CLASS='serviceWARNING'>%2.3f%% (%2.3f%%)</td><td CLASS='serviceUNKNOWN'>%2.3f%% (%2.3f%%)</td><td class='serviceCRITICAL'>%2.3f%% (%2.3f%%)</td><td class='data%s'>%2.3f%%</td><td class='service%s'>%2.3f%% (%2.3f%%)</td><td class='data%s'>%2.3f%%</td></tr>\n",
+				sg_stat_array[i].average_percent_time_ok,
+				sg_stat_array[i].average_percent_time_ok_known,
+				sg_stat_array[i].average_percent_time_warning,
+				sg_stat_array[i].average_percent_time_warning_known,
+				sg_stat_array[i].average_percent_time_unknown,
+				sg_stat_array[i].average_percent_time_unknown_known,
+				sg_stat_array[i].average_percent_time_critical,
+				sg_stat_array[i].average_percent_time_critical_known,
+				bgclass[odd],
+				sg_stat_array[i].average_percent_time_indeterminate,
+				(char *)(((int)(sg_stat_array[i].sla_actual*1000.0+0.5) < (int)(sg_stat_array[i].sg->sla_target*1000.0+0.5)) ? "CRITICAL" : "OK"),
+				sg_stat_array[i].sla_actual,
+				(sg_stat_array[i].average_percent_time_ok_known+sg_stat_array[i].average_percent_time_warning_known+sg_stat_array[i].average_percent_time_unknown_known+sg_stat_array[i].average_percent_time_critical_known),
+				bgclass[odd],
+				sg_stat_array[i].sg->sla_target);
+			}
+		else if(output_format==CSV_OUTPUT){
+			printf("%s,%2.3f,%2.3f,%2.3f,%2.3f,%2.3f,%2.3f,%2.3f,%2.3f,%2.3f,%2.3f,%2.3f,%2.3f,%s\n",
+				sg_stat_array[i].sg->group_name,
+				sg_stat_array[i].average_percent_time_ok,
+				sg_stat_array[i].average_percent_time_ok_known,
+				sg_stat_array[i].average_percent_time_warning,
+				sg_stat_array[i].average_percent_time_warning_known,
+				sg_stat_array[i].average_percent_time_unknown,
+				sg_stat_array[i].average_percent_time_unknown_known,
+				sg_stat_array[i].average_percent_time_critical,
+				sg_stat_array[i].average_percent_time_critical_known,
+				sg_stat_array[i].average_percent_time_indeterminate,
+				sg_stat_array[i].sla_actual,
+				(sg_stat_array[i].average_percent_time_ok_known+sg_stat_array[i].average_percent_time_warning_known+sg_stat_array[i].average_percent_time_unknown_known+sg_stat_array[i].average_percent_time_critical_known),
+				sg_stat_array[i].sg->sla_target,
+				(char *)(((int)(sg_stat_array[i].sla_actual*1000.0+0.5) < (int)(sg_stat_array[i].sg->sla_target*1000.0+0.5)) ? "NO" : "YES"));
+			}
+
+		if ((int)(sg_stat_array[i].sla_actual*1000.0+0.5) >= (int)(sg_stat_array[i].sg->sla_target*1000.0+0.5))
+			totalWithinSLA++;
+
+		odd ^= 1;
+		}
+
+	/* Convert the running totals into averages */
+	allGTs.average_percent_time_ok /= totalGroups;
+	allGTs.average_percent_time_ok_known /= totalGroups;
+	allGTs.average_percent_time_unknown /= totalGroups;
+	allGTs.average_percent_time_unknown_known /= totalGroups;
+	allGTs.average_percent_time_warning /= totalGroups;
+	allGTs.average_percent_time_warning_known /= totalGroups;
+	allGTs.average_percent_time_critical /= totalGroups;
+	allGTs.average_percent_time_critical_known /= totalGroups;
+	allGTs.average_percent_time_indeterminate /= totalGroups;
+	allGTs.sla_actual /= totalGroups;
+	allGTs.sla_target /= totalGroups;
+
+	/* Display the averages-of-the-averages row */
+	if(output_format==HTML_OUTPUT){
+		printf("<tr CLASS='data%s'><td CLASS='data'><center><b>",bgclass[odd]);
+		printf("Average (%u Groups)", totalGroups);
+		printf("</b></center></td><td CLASS='serviceOK'>%2.3f%% (%2.3f%%)</td><td CLASS='serviceWARNING'>%2.3f%% (%2.3f%%)</td><td CLASS='serviceUNKNOWN'>%2.3f%% (%2.3f%%)</td><td class='serviceCRITICAL'>%2.3f%% (%2.3f%%)</td><td class='data%s'>%2.3f%%</td><td class='data%s'>%2.3f%%</td><td class='data%s'>%2.3f%%</td></tr>\n",
+			allGTs.average_percent_time_ok,
+			allGTs.average_percent_time_ok_known,
+			allGTs.average_percent_time_warning,
+			allGTs.average_percent_time_warning_known,
+			allGTs.average_percent_time_unknown,
+			allGTs.average_percent_time_unknown_known,
+			allGTs.average_percent_time_critical,
+			allGTs.average_percent_time_critical_known,
+			bgclass[odd],
+			allGTs.average_percent_time_indeterminate,
+			bgclass[odd],
+			allGTs.sla_actual,
+			bgclass[odd],
+			allGTs.sla_target);
+
+		printf("</table>\n");
+		printf("</DIV>\n");
+		}
+
+	/* Compute statistical SLA summary */
+	totalOutOfSLA = totalGroups - totalWithinSLA;
+	percentWithinSLA = ((double)totalWithinSLA / (double)totalGroups) * 100.0;
+	percentOutOfSLA = 100.0 - percentWithinSLA;
+	odd ^= 1;
+	
+
+	/* Display the SLA statistics summary row */
+	if(output_format==HTML_OUTPUT){
+		printf("<BR><DIV ALIGN=CENTER>\n");
+		printf("<TABLE BORDER=0 WIDTH='100%%' CLASS='data'>\n");
+		printf("<tr CLASS='data%s'>", bgclass[odd]);
+		printf("<td class='data%s'><center><b>Total Groups: &nbsp&nbsp&nbsp %u</b></center></td>", bgclass[odd], totalGroups);
+		printf("<td class='data%s'><center><b>Within SLA: &nbsp&nbsp&nbsp %u &nbsp&nbsp (%2.3f%%)</b></center></td>", bgclass[odd], totalWithinSLA, percentWithinSLA);
+		printf("<td class='data%s'><center><b>Outside SLA: &nbsp&nbsp&nbsp %u &nbsp&nbsp (%2.3f%%)</b></center></td>", bgclass[odd], totalOutOfSLA, percentOutOfSLA);
+		printf("</TR>");
+
+		printf("</table>\n");
+		printf("</DIV>\n");
+		}
+
+	return;
+	}
+
+
+/* display availability summary for a specific servicegroup */
+int display_specific_servicegroup_availability_summary(servicegroup *sg, servicegroup_stats *psgs){
+	unsigned long total_time;
+	unsigned long time_determinate;
+	unsigned long time_indeterminate;
+	avail_subject *temp_subject;
+	double percent_time_up=0.0;
+	double percent_time_down=0.0;
+	double percent_time_unreachable=0.0;
+	double percent_time_up_known=0.0;
+	double percent_time_down_known=0.0;
+	double percent_time_unreachable_known=0.0;
+	double percent_time_indeterminate=0.0;
+	double percent_time_ok=0.0;
+	double percent_time_warning=0.0;
+	double percent_time_unknown=0.0;
+	double percent_time_critical=0.0;
+	double percent_time_ok_known=0.0;
+	double percent_time_warning_known=0.0;
+	double percent_time_unknown_known=0.0;
+	double percent_time_critical_known=0.0;
+
+	double percent_time_critical_scheduled=0.0;
+	double percent_time_critical_unscheduled=0.0;
+	double percent_time_critical_scheduled_known=0.0;
+	double percent_time_critical_unscheduled_known=0.0;
+
+	double percent_time_unknown_scheduled=0.0;
+	double percent_time_unknown_unscheduled=0.0;
+	double percent_time_unknown_scheduled_known=0.0;
+	double percent_time_unknown_unscheduled_known=0.0;
+
+	double percent_time_warning_scheduled=0.0;
+	double percent_time_warning_unscheduled=0.0;
+	double percent_time_warning_scheduled_known=0.0;
+	double percent_time_warning_unscheduled_known=0.0;
+
+	double percent_time_ok_scheduled=0.0;
+	double percent_time_ok_unscheduled=0.0;
+	double percent_time_ok_scheduled_known=0.0;
+	double percent_time_ok_unscheduled_known=0.0;
+
+	double average_percent_time_up=0.0;
+	double average_percent_time_up_known=0.0;
+	double average_percent_time_down=0.0;
+	double average_percent_time_down_known=0.0;
+	double average_percent_time_unreachable=0.0;
+	double average_percent_time_unreachable_known=0.0;
+	double average_percent_time_ok=0.0;
+	double average_percent_time_ok_known=0.0;
+	double average_percent_time_unknown=0.0;
+	double average_percent_time_unknown_known=0.0;
+	double average_percent_time_warning=0.0;
+	double average_percent_time_warning_known=0.0;
+	double average_percent_time_critical=0.0;
+	double average_percent_time_critical_known=0.0;
+	double average_percent_time_indeterminate=0.0;
+	double percent_time_indeterminate_scheduled=0.0;
+	double percent_time_indeterminate_unscheduled=0.0;
+	double percent_time_indeterminate_notrunning=0.0;
+	double percent_time_indeterminate_nodata=0.0;
+
+	int current_subject=0;
+
+	host *temp_host;
+	service *temp_service;
+
+	if(sg==NULL)
+		return 0;
+
+	/* the user isn't authorized to view this servicegroup */
+	if(is_authorized_for_servicegroup(sg,&current_authdata)==FALSE)
+		return 0;
+
+	/* calculate total time during period based on timeperiod used for reporting */
+	total_time=calculate_total_time(t1,t2);
+
+	/* Iterate through all hosts that are members of this service group */
+	for(temp_subject=subject_list;temp_subject!=NULL;temp_subject=temp_subject->next){
+
+		if(temp_subject->type!=HOST_SUBJECT)
+			continue;
+
+		temp_host=find_host(temp_subject->host_name);
+		if(temp_host==NULL)
+			continue;
+
+		if(is_host_member_of_servicegroup(sg,temp_host)==FALSE)
+			continue;
+
+		current_subject++;
+
+		/* reset variables */
+		percent_time_up=0.0;
+		percent_time_down=0.0;
+		percent_time_unreachable=0.0;
+		percent_time_indeterminate=0.0;
+		percent_time_up_known=0.0;
+		percent_time_down_known=0.0;
+		percent_time_unreachable_known=0.0;
+
+		time_determinate=temp_subject->time_up+temp_subject->time_down+temp_subject->time_unreachable;
+		time_indeterminate=total_time-time_determinate;
+	
+		if(total_time>0){
+			percent_time_up=(double)(((double)temp_subject->time_up*100.0)/(double)total_time);
+			percent_time_down=(double)(((double)temp_subject->time_down*100.0)/(double)total_time);
+			percent_time_unreachable=(double)(((double)temp_subject->time_unreachable*100.0)/(double)total_time);
+			percent_time_indeterminate=(double)(((double)time_indeterminate*100.0)/(double)total_time);
+			if(time_determinate>0){
+				percent_time_up_known=(double)(((double)temp_subject->time_up*100.0)/(double)time_determinate);
+				percent_time_down_known=(double)(((double)temp_subject->time_down*100.0)/(double)time_determinate);
+				percent_time_unreachable_known=(double)(((double)temp_subject->time_unreachable*100.0)/(double)time_determinate);
+	                        }
+		        }
+
+		get_running_average(&average_percent_time_up,percent_time_up,current_subject);
+		get_running_average(&average_percent_time_up_known,percent_time_up_known,current_subject);
+		get_running_average(&average_percent_time_down,percent_time_down,current_subject);
+		get_running_average(&average_percent_time_down_known,percent_time_down_known,current_subject);
+		get_running_average(&average_percent_time_unreachable,percent_time_unreachable,current_subject);
+		get_running_average(&average_percent_time_unreachable_known,percent_time_unreachable_known,current_subject);
+		get_running_average(&average_percent_time_indeterminate,percent_time_indeterminate,current_subject);
+                }
+
+	current_subject=0;
+	average_percent_time_indeterminate=0.0;
+
+	/* Iterate through all services within this service group */
+	for(temp_subject=subject_list;temp_subject!=NULL;temp_subject=temp_subject->next){
+
+		if(temp_subject->type!=SERVICE_SUBJECT)
+			continue;
+
+		temp_service=find_service(temp_subject->host_name,temp_subject->service_description);
+		if(temp_service==NULL)
+			continue;
+
+		if(is_service_member_of_servicegroup(sg,temp_service)==FALSE)
 			continue;
 
 		current_subject++;
@@ -3764,75 +4819,95 @@
 
 		/* initialize values */
 		percent_time_ok=0.0;
+		percent_time_ok_scheduled=0.0;
+		percent_time_ok_unscheduled=0.0;
 		percent_time_warning=0.0;
+		percent_time_warning_scheduled=0.0;
+		percent_time_warning_unscheduled=0.0;
 		percent_time_unknown=0.0;
+		percent_time_unknown_scheduled=0.0;
+		percent_time_unknown_unscheduled=0.0;
 		percent_time_critical=0.0;
+		percent_time_critical_scheduled=0.0;
+		percent_time_critical_unscheduled=0.0;
 		percent_time_indeterminate=0.0;
+		percent_time_indeterminate_scheduled=0.0;
+		percent_time_indeterminate_unscheduled=0.0;
+		percent_time_indeterminate_notrunning=0.0;
+		percent_time_indeterminate_nodata=0.0;
 		percent_time_ok_known=0.0;
+		percent_time_ok_scheduled_known=0.0;
+		percent_time_ok_unscheduled_known=0.0;
 		percent_time_warning_known=0.0;
+		percent_time_warning_scheduled_known=0.0;
+		percent_time_warning_unscheduled_known=0.0;
 		percent_time_unknown_known=0.0;
+		percent_time_unknown_scheduled_known=0.0;
+		percent_time_unknown_unscheduled_known=0.0;
 		percent_time_critical_known=0.0;
+		percent_time_critical_scheduled_known=0.0;
+		percent_time_critical_unscheduled_known=0.0;
 
 		if(total_time>0){
 			percent_time_ok=(double)(((double)temp_subject->time_ok*100.0)/(double)total_time);
+			percent_time_ok_scheduled=(double)(((double)temp_subject->scheduled_time_ok*100.0)/(double)total_time);
+			percent_time_ok_unscheduled=percent_time_ok-percent_time_ok_scheduled;
 			percent_time_warning=(double)(((double)temp_subject->time_warning*100.0)/(double)total_time);
+			percent_time_warning_scheduled=(double)(((double)temp_subject->scheduled_time_unknown*100.0)/(double)total_time);
+			percent_time_warning_unscheduled=percent_time_warning-percent_time_warning_scheduled;
 			percent_time_unknown=(double)(((double)temp_subject->time_unknown*100.0)/(double)total_time);
+			percent_time_unknown_scheduled=(double)(((double)temp_subject->scheduled_time_unknown*100.0)/(double)total_time);
+			percent_time_unknown_unscheduled=percent_time_unknown-percent_time_unknown_scheduled;
 			percent_time_critical=(double)(((double)temp_subject->time_critical*100.0)/(double)total_time);
+			percent_time_critical_scheduled=(double)(((double)temp_subject->scheduled_time_critical*100.0)/(double)total_time);
+			percent_time_critical_unscheduled=percent_time_critical-percent_time_critical_scheduled;
 			percent_time_indeterminate=(double)(((double)time_indeterminate*100.0)/(double)total_time);
+			percent_time_indeterminate_scheduled=(double)(((double)temp_subject->scheduled_time_indeterminate*100.0)/(double)total_time);
+			percent_time_indeterminate_unscheduled=percent_time_indeterminate-percent_time_indeterminate_scheduled;
+			percent_time_indeterminate_notrunning=(double)(((double)temp_subject->time_indeterminate_notrunning*100.0)/(double)total_time);
+			percent_time_indeterminate_nodata=(double)(((double)temp_subject->time_indeterminate_nodata*100.0)/(double)total_time);
 			if(time_determinate>0){
 				percent_time_ok_known=(double)(((double)temp_subject->time_ok*100.0)/(double)time_determinate);
+				percent_time_ok_scheduled_known=(double)(((double)temp_subject->scheduled_time_ok*100.0)/(double)time_determinate);
+				percent_time_ok_unscheduled_known=percent_time_ok_known-percent_time_ok_scheduled_known;
 				percent_time_warning_known=(double)(((double)temp_subject->time_warning*100.0)/(double)time_determinate);
+				percent_time_warning_scheduled_known=(double)(((double)temp_subject->scheduled_time_warning*100.0)/(double)time_determinate);
+				percent_time_warning_unscheduled_known=percent_time_warning_known-percent_time_warning_scheduled_known;
 				percent_time_unknown_known=(double)(((double)temp_subject->time_unknown*100.0)/(double)time_determinate);
+				percent_time_unknown_scheduled_known=(double)(((double)temp_subject->scheduled_time_unknown*100.0)/(double)time_determinate);
+				percent_time_unknown_unscheduled_known=percent_time_unknown_known-percent_time_unknown_scheduled_known;
 				percent_time_critical_known=(double)(((double)temp_subject->time_critical*100.0)/(double)time_determinate);
+				percent_time_critical_scheduled_known=(double)(((double)temp_subject->scheduled_time_critical*100.0)/(double)time_determinate);
+				percent_time_critical_unscheduled_known=percent_time_critical_known-percent_time_critical_scheduled_known;
 	                        }
                         }
 
-		if(odd){
-			odd=0;
-			bgclass="Odd";
-	                }
-		else{
-			odd=1;
-			bgclass="Even";
-	                }
-
-		printf("<tr CLASS='data%s'><td CLASS='data%s'>",bgclass,bgclass);
-		if(strcmp(temp_subject->host_name,last_host))
-			host_report_url(temp_subject->host_name,temp_subject->host_name);
-		printf("</td><td CLASS='data%s'>",bgclass);
-		service_report_url(temp_subject->host_name,temp_subject->service_description,temp_subject->service_description);
-		printf("</td><td CLASS='serviceOK'>%2.3f%% (%2.3f%%)</td><td CLASS='serviceWARNING'>%2.3f%% (%2.3f%%)</td><td CLASS='serviceUNKNOWN'>%2.3f%% (%2.3f%%)</td><td class='serviceCRITICAL'>%2.3f%% (%2.3f%%)</td><td class='data%s'>%2.3f%%</td></tr>\n",percent_time_ok,percent_time_ok_known,percent_time_warning,percent_time_warning_known,percent_time_unknown,percent_time_unknown_known,percent_time_critical,percent_time_critical_known,bgclass,percent_time_indeterminate);
-
-		strncpy(last_host,temp_subject->host_name,sizeof(last_host)-1);
-		last_host[sizeof(last_host)-1]='\x0';
 
-		get_running_average(&average_percent_time_ok,percent_time_ok,current_subject);
-		get_running_average(&average_percent_time_ok_known,percent_time_ok_known,current_subject);
-		get_running_average(&average_percent_time_unknown,percent_time_unknown,current_subject);
-		get_running_average(&average_percent_time_unknown_known,percent_time_unknown_known,current_subject);
-		get_running_average(&average_percent_time_warning,percent_time_warning,current_subject);
-		get_running_average(&average_percent_time_warning_known,percent_time_warning_known,current_subject);
-		get_running_average(&average_percent_time_critical,percent_time_critical,current_subject);
-		get_running_average(&average_percent_time_critical_known,percent_time_critical_known,current_subject);
+		get_running_average(&average_percent_time_ok,percent_time_ok_unscheduled_known,current_subject);
+		get_running_average(&average_percent_time_ok_known,percent_time_ok_scheduled_known,current_subject);
+		get_running_average(&average_percent_time_unknown,percent_time_unknown_unscheduled_known,current_subject);
+		get_running_average(&average_percent_time_unknown_known,percent_time_unknown_scheduled_known,current_subject);
+		get_running_average(&average_percent_time_warning,percent_time_warning_unscheduled_known,current_subject);
+		get_running_average(&average_percent_time_warning_known,percent_time_warning_scheduled_known,current_subject);
+		get_running_average(&average_percent_time_critical,percent_time_critical_unscheduled_known,current_subject);
+		get_running_average(&average_percent_time_critical_known,percent_time_critical_scheduled_known,current_subject);
 		get_running_average(&average_percent_time_indeterminate,percent_time_indeterminate,current_subject);
 	        }
 
-	/* display average stats */
-	if(odd){
-		odd=0;
-		bgclass="Odd";
-	        }
-	else{
-		odd=1;
-		bgclass="Even";
-	        }
-
-	printf("<tr CLASS='data%s'><td CLASS='data%s' colspan='2'>Average</td><td CLASS='serviceOK'>%2.3f%% (%2.3f%%)</td><td CLASS='serviceWARNING'>%2.3f%% (%2.3f%%)</td><td CLASS='serviceUNKNOWN'>%2.3f%% (%2.3f%%)</td><td class='serviceCRITICAL'>%2.3f%% (%2.3f%%)</td><td class='data%s'>%2.3f%%</td></tr>\n",bgclass,bgclass,average_percent_time_ok,average_percent_time_ok_known,average_percent_time_warning,average_percent_time_warning_known,average_percent_time_unknown,average_percent_time_unknown_known,average_percent_time_critical,average_percent_time_critical_known,bgclass,average_percent_time_indeterminate);
-
-	printf("</table>\n");
-	printf("</DIV>\n");
+	/* Save this group's averages */
+	psgs->average_percent_time_ok = average_percent_time_ok;
+	psgs->average_percent_time_ok_known = average_percent_time_ok_known;
+	psgs->average_percent_time_unknown = average_percent_time_unknown;
+	psgs->average_percent_time_unknown_known = average_percent_time_unknown_known;
+	psgs->average_percent_time_warning = average_percent_time_warning;
+	psgs->average_percent_time_warning_known = average_percent_time_warning_known;
+	psgs->average_percent_time_critical = average_percent_time_critical;
+	psgs->average_percent_time_critical_known = average_percent_time_critical_known;
+	psgs->average_percent_time_indeterminate = average_percent_time_indeterminate;
+	psgs->sla_actual = 100.0 - average_percent_time_critical;
+	psgs->sla_target = sg->sla_target;
 
-	return;
+	return 1;
         }
 
 
@@ -4510,6 +5585,8 @@
 
 		printf("<DIV ALIGN=CENTER CLASS='dataTitle'>Service State Breakdowns:</DIV>\n");
 #ifdef USE_TRENDS
+		printf("<BR><DIV ALIGN=CENTER CLASS='dataTitle'>%s</DIV>\n", temp_service->service_check_command);
+
 		printf("<p align='center'>\n");
 		printf("<a href='%s?host=%s",TRENDS_CGI,url_encode(host_name));
 		printf("&service=%s&t1=%lu&t2=%lu&includesoftstates=%s&assumestateretention=%s&assumeinitialstates=%s&assumestatesduringnotrunning=%s&initialassumedservicestate=%d&backtrack=%d'>",url_encode(svc_description),t1,t2,(include_soft_states==TRUE)?"yes":"no",(assume_state_retention==TRUE)?"yes":"no",(assume_initial_states==TRUE)?"yes":"no",(assume_states_during_notrunning==TRUE)?"yes":"no",initial_assumed_service_state,backtrack_archives);
@@ -4576,7 +5653,7 @@
 
 			printf("<DIV ALIGN=CENTER>\n");
 			printf("<TABLE BORDER=0 CLASS='data'>\n");
-			printf("<TR><TH CLASS='data'>Host</TH><TH CLASS='data'>Service</TH><TH CLASS='data'>%% Time OK</TH><TH CLASS='data'>%% Time Warning</TH><TH CLASS='data'>%% Time Unknown</TH><TH CLASS='data'>%% Time Critical</TH><TH CLASS='data'>%% Time Undetermined</TH></TR>\n");
+			printf("<TR><TH CLASS='data'>Host</TH><TH CLASS='data'>Service</TH><TH CLASS='data'>%% Time OK</TH><TH CLASS='data'>%% Time Warning</TH><TH CLASS='data'>%% Time Unknown</TH><TH CLASS='data'>%% Time Critical</TH><TH CLASS='data'>%% Time Undetermined</TH><TH CLASS='data'>Service Check Parameters</TH></TR>\n");
 		        }
 		else if(output_format==CSV_OUTPUT){
 			printf("HOST_NAME, SERVICE_DESCRIPTION,");
@@ -4589,7 +5666,7 @@
 
 			printf(" TIME_CRITICAL_SCHEDULED, PERCENT_TIME_CRITICAL_SCHEDULED, PERCENT_KNOWN_TIME_CRITICAL_SCHEDULED, TIME_CRITICAL_UNSCHEDULED, PERCENT_TIME_CRITICAL_UNSCHEDULED, PERCENT_KNOWN_TIME_CRITICAL_UNSCHEDULED, TOTAL_TIME_CRITICAL, PERCENT_TOTAL_TIME_CRITICAL, PERCENT_KNOWN_TIME_CRITICAL,");
 
-			printf(" TIME_UNDETERMINED_NOT_RUNNING, PERCENT_TIME_UNDETERMINED_NOT_RUNNING, TIME_UNDETERMINED_NO_DATA, PERCENT_TIME_UNDETERMINED_NO_DATA, TOTAL_TIME_UNDETERMINED, PERCENT_TOTAL_TIME_UNDETERMINED\n");
+			printf(" TIME_UNDETERMINED_NOT_RUNNING, PERCENT_TIME_UNDETERMINED_NOT_RUNNING, TIME_UNDETERMINED_NO_DATA, PERCENT_TIME_UNDETERMINED_NO_DATA, TOTAL_TIME_UNDETERMINED, PERCENT_TOTAL_TIME_UNDETERMINED, CHECK_COMMAND\n");
 		        }
 		   
 
@@ -4695,7 +5772,7 @@
 					host_report_url(temp_subject->host_name,temp_subject->host_name);
 				printf("</td><td CLASS='data%s'>",bgclass);
 				service_report_url(temp_subject->host_name,temp_subject->service_description,temp_subject->service_description);
-				printf("</td><td CLASS='serviceOK'>%2.3f%% (%2.3f%%)</td><td CLASS='serviceWARNING'>%2.3f%% (%2.3f%%)</td><td CLASS='serviceUNKNOWN'>%2.3f%% (%2.3f%%)</td><td class='serviceCRITICAL'>%2.3f%% (%2.3f%%)</td><td class='data%s'>%2.3f%%</td></tr>\n",percent_time_ok,percent_time_ok_known,percent_time_warning,percent_time_warning_known,percent_time_unknown,percent_time_unknown_known,percent_time_critical,percent_time_critical_known,bgclass,percent_time_indeterminate);
+				printf("</td><td CLASS='serviceOK'>%2.3f%% (%2.3f%%)</td><td CLASS='serviceWARNING'>%2.3f%% (%2.3f%%)</td><td CLASS='serviceUNKNOWN'>%2.3f%% (%2.3f%%)</td><td class='serviceCRITICAL'>%2.3f%% (%2.3f%%)</td><td class='data%s'>%2.3f%%</td><td class='data%s'>%s</td></tr>\n",percent_time_ok,percent_time_ok_known,percent_time_warning,percent_time_warning_known,percent_time_unknown,percent_time_unknown_known,percent_time_critical,percent_time_critical_known,bgclass,percent_time_indeterminate,bgclass,temp_service->service_check_command);
 			        }
 			else if(output_format==CSV_OUTPUT){
 
@@ -4715,7 +5792,9 @@
 				printf(" %lu, %2.3f%%, %2.3f%%, %lu, %2.3f%%, %2.3f%%, %lu, %2.3f%%, %2.3f%%,",temp_subject->scheduled_time_critical,percent_time_critical_scheduled,percent_time_critical_scheduled_known,temp_subject->time_critical-temp_subject->scheduled_time_critical,percent_time_critical_unscheduled,percent_time_critical_unscheduled_known,temp_subject->time_critical,percent_time_critical,percent_time_critical_known);
 
 				/* indeterminate times */
-				printf(" %lu, %2.3f%%, %lu, %2.3f%%, %lu, %2.3f%%\n",temp_subject->time_indeterminate_notrunning,percent_time_indeterminate_notrunning,temp_subject->time_indeterminate_nodata,percent_time_indeterminate_nodata,time_indeterminate,percent_time_indeterminate);
+				printf(" %lu, %2.3f%%, %lu, %2.3f%%, %lu, %2.3f%%,",temp_subject->time_indeterminate_notrunning,percent_time_indeterminate_notrunning,temp_subject->time_indeterminate_nodata,percent_time_indeterminate_nodata,time_indeterminate,percent_time_indeterminate);
+				/* check command */
+				printf(" \"%s\"\n",temp_service->service_check_command);
 			        }
 
 			strncpy(last_host,temp_subject->host_name,sizeof(last_host)-1);
@@ -4743,7 +5822,7 @@
 				odd=1;
 				bgclass="Even";
       	                        }
-			printf("<tr CLASS='data%s'><td CLASS='data%s' colspan='2'>Average</td><td CLASS='serviceOK'>%2.3f%% (%2.3f%%)</td><td CLASS='serviceWARNING'>%2.3f%% (%2.3f%%)</td><td CLASS='serviceUNKNOWN'>%2.3f%% (%2.3f%%)</td><td class='serviceCRITICAL'>%2.3f%% (%2.3f%%)</td><td class='data%s'>%2.3f%%</td></tr>\n",bgclass,bgclass,average_percent_time_ok,average_percent_time_ok_known,average_percent_time_warning,average_percent_time_warning_known,average_percent_time_unknown,average_percent_time_unknown_known,average_percent_time_critical,average_percent_time_critical_known,bgclass,average_percent_time_indeterminate);
+			printf("<tr CLASS='data%s'><td CLASS='data%s' colspan='2'>Average</td><td CLASS='serviceOK'>%2.3f%% (%2.3f%%)</td><td CLASS='serviceWARNING'>%2.3f%% (%2.3f%%)</td><td CLASS='serviceUNKNOWN'>%2.3f%% (%2.3f%%)</td><td class='serviceCRITICAL'>%2.3f%% (%2.3f%%)</td><td class='data%s'>%2.3f%%</td><td class='data%s'></td></tr>\n",bgclass,bgclass,average_percent_time_ok,average_percent_time_ok_known,average_percent_time_warning,average_percent_time_warning_known,average_percent_time_unknown,average_percent_time_unknown_known,average_percent_time_critical,average_percent_time_critical_known,bgclass,average_percent_time_indeterminate,bgclass);
 
 			printf("</table>\n");
 			printf("</DIV>\n");
@@ -4803,6 +5882,71 @@
 	return;
         }
 
+void service_detail_url(char *sd, char *label){
+
+	printf("<a href='%s?",AVAIL_CGI);
+	printf("servicegroup=%s",url_encode(sd));
+	if(show_log_entries==TRUE)
+		printf("&show_log_entries");
+	printf("&t1=%lu&t2=%lu", t1, t2);
+	printf("&smon=%d&sday=%d&syear=%d&shour=%d&smin=%d&ssec=%d",start_month,start_day,start_year,start_hour,start_minute,start_second);
+	printf("&emon=%d&eday=%d&eyear=%d&ehour=%d&emin=%d&esec=%d",end_month,end_day,end_year,end_hour,end_minute,end_second);
+	printf("&timeperiod=%s",TimePeriods[timeperiod_type]);
+	printf("&assumeinitialstates=%s",(assume_initial_states==TRUE)?"yes":"no");
+	printf("&assumestateretention=%s",(assume_state_retention==TRUE)?"yes":"no");
+	printf("&assumestatesduringnotrunning=%s",(assume_states_during_notrunning==TRUE)?"yes":"no");
+	printf("&initialassumedhoststate=%d",initial_assumed_host_state);
+	printf("&initialassumedservicestate=%d",initial_assumed_service_state);
+	printf("&backtrack=%d",backtrack_archives);
+	printf("&sortcol=%d",sort_column);
+	printf("&sortdir=%d",sort_direction);
+	printf("'>%s</a>",label);
+
+	return;
+        }
+
+void avail_sort_url(char *sd, char *label, int scol){
+
+	char temp_url[1024];
+
+	sprintf(temp_url, "%s?servicegroup=%s&t1=%lu&t2=%lu&smon=%d&sday=%d&syear=%d&shour=%d&smin=%d&ssec=%d&emon=%d&eday=%d&eyear=%d&ehour=%d&emin=%d&esec=%d&timeperiod=%s&assumeinitialstates=%s&assumestateretention=%s&assumestatesduringnotrunning=%s&includesoftstates=%s&initialassumedhoststate=%d&initialassumedservicestate=%d&backtrack=%d",
+		AVAIL_CGI,
+		url_encode(sd),
+		t1, t2,
+		start_month,start_day,start_year,start_hour,start_minute,start_second,
+		end_month,end_day,end_year,end_hour,end_minute,end_second,
+		TimePeriods[timeperiod_type],
+		((assume_initial_states==TRUE)?"yes":"no"),
+		((assume_state_retention==TRUE)?"yes":"no"),
+		((assume_states_during_notrunning==TRUE)?"yes":"no"),
+		((include_soft_states==TRUE)?"yes":"no"),
+		initial_assumed_host_state,
+		initial_assumed_service_state,
+		backtrack_archives);
+
+	if(show_log_entries==TRUE)
+		strcat(temp_url,"&show_log_entries");
+
+	printf("<TH CLASS='data'>%s&nbsp;<A HREF='%s&sortcol=%d&sortdir=%d'><IMG SRC='%s%s' BORDER=0 ALT='Sort by %s (ascending)' TITLE='Sort by %s (ascending)'></A><A HREF='%s&sortcol=%d&sortdir=%d'><IMG SRC='%s%s' BORDER=0 ALT='Sort by %s (descending)' TITLE='Sort by %s (descending)'></A></TH>",
+		label,
+		temp_url,
+		scol,
+		SORT_DIR_ASC,
+		url_images_path,
+		UP_ARROW_ICON,
+		label,
+		label,
+		temp_url,
+		scol,
+		SORT_DIR_DESC,
+		url_images_path,
+		DOWN_ARROW_ICON,
+		label,
+		label);
+
+	return;
+        }
+
 
 /* calculates running average */
 void get_running_average(double *running_average, double new_value, int current_item){
@@ -4870,3 +6014,172 @@
 	/* no timeperiod was selected */
 	return end_time-start_time;
         }
+
+static void show_subj_time (unsigned long t)
+{
+	unsigned long days, hours, mins, secs;
+
+	days  = t/86400;
+	t -= (days*86400);
+
+	hours = t/3600;
+	t -= (hours*3600);
+
+	mins  = t/60;
+	t -= (mins*60);
+
+	secs  = t;
+
+	printf("  %02lu:%02lu:%02lu:%02lu", days, hours, mins, secs);
+}
+
+static void dump_subjects (void)
+{
+	avail_subject *temp_subject;
+	archived_state *temp_as;
+	struct tm *bobbaloo;
+	char *cp;
+	static char *Events[] = {
+		"NO_DATA",
+		"PROGRAM_END",
+		"PROGRAM_START",
+		"HOST_UP",
+		"HOST_DOWN",
+		"HOST_UNREACHABLE",
+		"SVC_OK",
+		"SVC_UNKNOWN",
+		"SVC_WARNING",
+		"SVC_CRITICAL",
+		"SVC_DOWNTIME_START",
+		"SVC_DOWNTIME_END",
+		"HOST_DOWNTIME_START",
+		"HOST_DOWNTIME_END"
+	};
+
+	// Dump each subject in the subject list
+	printf("<PRE>\n");
+	for (temp_subject = subject_list; temp_subject; temp_subject=temp_subject->next)
+	{
+		// Dump subject description and statistics
+		printf("<PRE>\n\n");
+		if (temp_subject->type == HOST_SUBJECT)
+		{
+			printf("Host Name                 Up          Down      Unreachable     S-Up        S-Down      S-Unreach     S-Indet      No-Data      Not-Run  \n");
+			printf("--------------------  -----------  -----------  -----------  -----------  -----------  -----------  -----------  -----------  -----------\n");
+			printf("%-20.20s", temp_subject->host_name);
+				show_subj_time(temp_subject->time_up);
+				show_subj_time(temp_subject->time_down);
+				show_subj_time(temp_subject->time_unreachable);
+				show_subj_time(temp_subject->scheduled_time_up);
+				show_subj_time(temp_subject->scheduled_time_down);
+				show_subj_time(temp_subject->scheduled_time_unreachable);
+				show_subj_time(temp_subject->scheduled_time_indeterminate);
+				show_subj_time(temp_subject->time_indeterminate_nodata);
+				show_subj_time(temp_subject->time_indeterminate_notrunning);
+		}
+		else	// Service
+		{
+			printf("Service Name              Ok          Warn       Critical      Unknown       S-Ok        S-Warn     S-Critical    S-Unknown     S-Indet      No-Data      Not-Run  \n");
+			printf("--------------------  -----------  -----------  -----------  -----------  -----------  -----------  -----------  -----------  -----------  -----------  -----------\n");
+			printf("%-20.20s", temp_subject->service_description);
+				show_subj_time(temp_subject->time_ok);
+				show_subj_time(temp_subject->time_warning);
+				show_subj_time(temp_subject->time_critical);
+				show_subj_time(temp_subject->time_unknown);
+				show_subj_time(temp_subject->scheduled_time_ok);
+				show_subj_time(temp_subject->scheduled_time_warning);
+				show_subj_time(temp_subject->scheduled_time_critical);
+				show_subj_time(temp_subject->scheduled_time_unknown);
+				show_subj_time(temp_subject->scheduled_time_indeterminate);
+				show_subj_time(temp_subject->time_indeterminate_nodata);
+				show_subj_time(temp_subject->time_indeterminate_notrunning);
+		}
+
+
+		// Dump subject's archived-state list
+		printf("\n\n\tArchived State List:\n");
+		printf("\t    Timestamp      Type         Event                Processed       Info\n");
+		printf("\t-----------------  ----  --------------------  --------------------  ----------------------------------------\n");
+		for (temp_as = temp_subject->as_list; temp_as; temp_as=temp_as->next)
+		{
+			bobbaloo = localtime(&(temp_as->time_stamp));
+			printf("\t%04d%02d%02d-%02d:%02d:%02d",
+				(bobbaloo->tm_year + 1900), (bobbaloo->tm_mon + 1), bobbaloo->tm_mday, bobbaloo->tm_hour, bobbaloo->tm_min, bobbaloo->tm_sec);
+
+			switch (temp_as->state_type)
+			{
+			case AS_SOFT_STATE:
+				cp = "Soft";
+				break;
+			case AS_HARD_STATE:
+				cp = "HARD";
+				break;
+			default:
+				cp = "";
+			}
+			printf("  %-4.4s", cp);
+
+			if (temp_as->entry_type == AS_CURRENT_STATE)
+				cp = "CURRENT_STATE";
+			else if (temp_as->entry_type > AS_HOST_DOWNTIME_END)
+				cp = "****";
+			else
+				cp = Events[temp_as->entry_type];
+			printf("  %-20.20s", cp);
+
+			if (temp_as->processed_state == AS_CURRENT_STATE)
+				cp = "CURRENT_STATE";
+			else if (temp_as->processed_state > AS_HOST_DOWNTIME_END)
+				cp = "****";
+			else
+				cp = Events[temp_as->processed_state];
+			printf("  %-20.20s", cp);
+
+			printf("  %-40.40s\n", (char *)((temp_as->state_info) ? temp_as->state_info : ""));
+		}
+
+		// Dump subject's scheduled-downtime list
+		printf("\n\tScheduled Downtime List:\n");
+		printf("\t    Timestamp      Type         Event                Processed       Info\n");
+		printf("\t-----------------  ----  --------------------  --------------------  ----------------------------------------\n");
+		for (temp_as = temp_subject->sd_list; temp_as; temp_as=temp_as->next)
+		{
+			bobbaloo = localtime(&(temp_as->time_stamp));
+			printf("\t%04d%02d%02d-%02d:%02d:%02d",
+				(bobbaloo->tm_year + 1900), (bobbaloo->tm_mon + 1), bobbaloo->tm_mday, bobbaloo->tm_hour, bobbaloo->tm_min, bobbaloo->tm_sec);
+
+			switch (temp_as->state_type)
+			{
+			case AS_SOFT_STATE:
+				cp = "Soft";
+				break;
+			case AS_HARD_STATE:
+				cp = "HARD";
+				break;
+			default:
+				cp = "";
+			}
+			printf("  %-4.4s", cp);
+
+			if (temp_as->entry_type == AS_CURRENT_STATE)
+				cp = "CURRENT_STATE";
+			else if (temp_as->entry_type > AS_HOST_DOWNTIME_END)
+				cp = "****";
+			else
+				cp = Events[temp_as->entry_type];
+			printf("  %-20.20s", cp);
+
+			if (temp_as->processed_state == AS_CURRENT_STATE)
+				cp = "CURRENT_STATE";
+			else if (temp_as->processed_state > AS_HOST_DOWNTIME_END)
+				cp = "****";
+			else
+				cp = Events[temp_as->processed_state];
+			printf("  %-20.20s", cp);
+
+			printf("  %-40.40s\n", (char *)((temp_as->state_info) ? temp_as->state_info : ""));
+		}
+	}
+	printf("</PRE>\n");
+}
+
